#include "renderer.h"

/* Initializes a renderer, given a window and renderer flags. */
void
renderer_init(SDL_Renderer** _x, SDL_Window* _w, s32 _f)
{
	*_x = SDL_CreateRenderer
	(
		_w,
		-1,
		_f
	);

	SDL_SetRenderDrawBlendMode(*_x, SDL_BLENDMODE_BLEND);
}

/* Frees a given renderer. */
void
renderer_free(SDL_Renderer* _x)
{
	SDL_DestroyRenderer(_x);
}

/* Clears the renderer. */
void
renderer_clear(SDL_Renderer* _x)
{
	SDL_RenderClear(_x);
}

/* Presents the renderer to the screen. */
void
renderer_present(SDL_Renderer* _x)
{
	SDL_RenderPresent(_x);
}

/* Gets the renderer's current draw color. */
SDL_Color
renderer_color_get(SDL_Renderer* _r)
{
	SDL_Color c;

	memset(&c, '\0', sizeof(SDL_Color));

	SDL_GetRenderDrawColor
	(
		_r, 
		&c.r, 
		&c.g, 
		&c.b, 
		&c.a
	);

	return c;
}

/* Sets the renderer's current draw color. */
void
renderer_color_set(SDL_Renderer* _x, SDL_Color _c)
{
	SDL_SetRenderDrawColor
	(
		_x,
		_c.r,
		_c.g,
		_c.b,
		_c.a
	);
}

/* Set's the renderer's viewport; e.g., where it's going to be drawing to. */
void
renderer_viewport_set(SDL_Renderer* _x, SDL_Rect _r)
{
	SDL_RenderSetViewport(_x, &_r);
}

/* Sets the renderer's viewport to the window's. */
void
renderer_viewport_window_set(SDL_Renderer* _x, SDL_Window* _w)
{
	SDL_Rect r;

	r = window_rect_get(_w);

	renderer_viewport_set(_x, r);
}
