#include "sprite.h"

/* Gets the sprite's current rectangle. */
SDL_Rect
sprite_rect_get(Sprite* _x)
{
	SDL_Rect r;

	memset(&r, '\0', sizeof(SDL_Rect));

        r.w = _x->w * _x->scale * _x->squash;
        r.h = _x->h * _x->scale * _x->stretch;

        switch (_x->origin)
        {
                case ORIGIN_TOP_LEFT:
                        r.x = _x->x;
                        r.y = _x->y;
                        break;
		/* FALLTHROUGH */
                case ORIGIN_MIDDLE:
                default:
                        r.x = _x->x - (r.w / 2);
                        r.y = _x->y - (r.h / 2);
                        break;
        }

	return r;
}

/* 
 * Initializes a sprite with an atlas, its origin, 
 * its width and its height. 
*/
void
sprite_atlas_init(Sprite* _x, Atlas* _a, OriginType _o, s32 _w, s32 _h)
{
	memset(_x, '\0', sizeof(Sprite));

	_x->atlas = *_a;
	_x->origin = _o;
	_x->w = _w;
	_x->h = _h;
	_x->rotation = SPRITE_ROTATION_DEFAULT;
	_x->frame = SPRITE_FRAME_DEFAULT;
	_x->scale = SPRITE_SCALE_DEFAULT;
	_x->squash = SPRITE_SQUASH_DEFAULT;
	_x->stretch = SPRITE_STRETCH_DEFAULT;
	_x->color = SPRITE_COLOR_DEFAULT;
	_x->flip = FLIP_NONE;
}

/* 
 * Initializes a sprite, given a texture, an origin, 
 * its width and its height. 
*/
void
sprite_init(Sprite* _x, SDL_Texture* _t, OriginType _o, s32 _w, s32 _h)
{
	Atlas a;

	atlas_init
	(
		&a, 
		_t, 
		ATLAS_ROWS_DEFAULT, 
		ATLAS_COLUMNS_DEFAULT, 
		_w,
		_h
	);

	sprite_atlas_init
	(
		_x, 
		&a, 
		_o, 
		_w,
		_h
	);
}

/* Draws the sprite. */
void
sprite_draw(Sprite* _x, SDL_Renderer* _r)
{
	SDL_Rect sourceRect;
	SDL_Rect destRect;
	s32 flip;

	destRect = sprite_rect_get(_x);

	if (_x->atlas.rows == 1 && _x->atlas.cols == 1)
	{
		sourceRect.x = 0;
		sourceRect.y = 0;
		sourceRect.w = texture_w_get(_x->atlas.texture);
		sourceRect.h = texture_h_get(_x->atlas.texture);
	}
	else
		sourceRect = atlas_source_rect_get(&_x->atlas, _x->frame);

	switch (_x->flip)
	{
		case FLIP_HORIZONTAL:
                	flip = SDL_FLIP_HORIZONTAL;
			break;
		case FLIP_VERTICAL:
                	flip = SDL_FLIP_VERTICAL;
			break;
		case FLIP_HORIZONTAL_VERTICAL:
			flip = SDL_FLIP_HORIZONTAL | SDL_FLIP_VERTICAL;
			break;
		/* FALLTHROUGH */
		case FLIP_NONE:
		default:
			flip = SDL_FLIP_NONE;
			break;
	}

	SDL_SetTextureAlphaMod
	(
		_x->atlas.texture, 
		_x->color.a
	);

	SDL_SetTextureColorMod
	(
		_x->atlas.texture,
		_x->color.r,
		_x->color.g,
		_x->color.b
	);
	
	SDL_RenderCopyEx
	(
		_r,
		_x->atlas.texture,
		&sourceRect,
		&destRect,
		_x->rotation,
		NULL,
		flip
	);
}
