#include "text.h"

static void text_surface_init(SDL_Surface** _x, SDL_Renderer* _r, TTF_Font* _f, SDL_Color _c, char* _s, s32 _w);
static void text_texture_init(SDL_Texture** _x, SDL_Renderer* _r, TTF_Font* _f, SDL_Color _c, char* _s, s32 _w);

/* 
 * Initializes a SDL_Surface with text, given
 * a renderer, a font, a color, a string, 
 * and a text wrap width.
*/
static void
text_surface_init
(
	SDL_Surface** _x, 
	SDL_Renderer* _r, 
	TTF_Font* _f, 
	SDL_Color _c, 
	char* _s, 
	s32 _w
)
{
	*_x = TTF_RenderText_Blended_Wrapped
	(
		_f, 
		_s, 
		_c, 
		_w
	);
}

/* 
 * Initializes a SDL_Texture with text,
 * given a renderer, a font, a color, a string, and a
 * text wrap width.
*/
static void
text_texture_init
(
 	SDL_Texture** _x,
	SDL_Renderer* _r, 
	TTF_Font* _f, 
	SDL_Color _c, 
	char* _s, 
	s32 _w
)
{
	SDL_Surface* s;

	text_surface_init
	(
	 	&s,
		_r, 
		_f, 
		_c, 
		_s, 
		_w
	);
	
	*_x = SDL_CreateTextureFromSurface(_r, s);

	SDL_FreeSurface(s);
}

/* Initializes a sprite with text, 
 * given a renderer, a font, a color, a string, an origin,
 * and a text wrap width.
*/
void
text_sprite_init
(
	Sprite* _x, 
	SDL_Renderer* _r, 
	TTF_Font* _f, 
	SDL_Color _c, 
	OriginType _o, 
	char* _s, 
	s32 _w
)
{
	SDL_Texture* t;

	text_texture_init
	(
		&t,
		_r, 
		_f,
		_c,
		_s,
		_w
	);

	sprite_init
	(
		_x,
		t,
		_o,
		texture_w_get(t),
		texture_h_get(t)
	);
}
