#include "window.h"

/* 
 * Initializes a window, given a window title, 
 * a width, a height, and window flags.
*/
void
window_init(SDL_Window** _x, char* _s, s32 _w, s32 _h, s32 _f)
{
	*_x =  SDL_CreateWindow
	(
		_s,
		SDL_WINDOWPOS_UNDEFINED,
		SDL_WINDOWPOS_UNDEFINED,
		_w,
		_h,
		_f
	);
}

/* Frees a given window. */
void
window_free(SDL_Window* _x)
{
	SDL_DestroyWindow(_x);
}

/* Gets the window's width. */
s32
window_w_get(SDL_Window* _x)
{
	s32 w;

	SDL_GetWindowSize(_x, &w, NULL);
	
	return w;
}

/* Get's the window's height. */
s32
window_h_get(SDL_Window* _x)
{
	s32 h;

	SDL_GetWindowSize(_x, NULL, &h);

	return h;
}

/* Gets the window's rectangle. */
SDL_Rect
window_rect_get(SDL_Window* _x)
{
	SDL_Rect r;

	memset(&r, '\0', sizeof(SDL_Rect));

	r.x = 0;
	r.y = 0;
	r.w = window_w_get(_x);
	r.h = window_h_get(_x);

	return r;
}

/* Sets video mode to fullscreen. */
void
window_fullscreen_enter(SDL_Window* _x)
{
	SDL_SetWindowFullscreen(_x, SDL_WINDOW_FULLSCREEN);
}

/* Entering fake fullscreen (desktop). */
void
window_fullscreen_desktop_enter(SDL_Window* _x)
{
	SDL_SetWindowFullscreen(_x, SDL_WINDOW_FULLSCREEN_DESKTOP);
}

/* Exiting fullscreen. */
void
window_fullscreen_exit(SDL_Window* _x)
{
	SDL_SetWindowFullscreen(_x, 0);
}
