/*
 * DESCRIPTION:
 * Fail ending code.
*/

#include "ending.h"

/* Initializes ending. */
void
ending_init(Ending* _e, Game* _g, EndingType _t)
{
	Sprite plusButtonSprite;
	Sprite endButtonSprite;

	memset(_e, '\0', sizeof(Intro));

	_e->g = _g;
	_e->type = _t;

	switch (_e->type)
	{
		case ENDING_FAIL:
			scene_init
			(
				&_e->sceneHandler, 
				_e->g, 
				(Scene*)ENDING_FAIL_SCENES,
				ENDING_FAIL_SCENE_COUNT
			);
			break;
		case ENDING_ONE:
			scene_init
			(
				&_e->sceneHandler, 
				_e->g, 
				(Scene*)ENDING_ONE_SCENES,
				ENDING_ONE_SCENE_COUNT
			);
			_e->isActualEnding = TRUE;
			break;
		case ENDING_TWO:
			scene_init
			(
				&_e->sceneHandler, 
				_e->g, 
				(Scene*)ENDING_TWO_SCENES,
				ENDING_TWO_SCENE_COUNT
			);
			_e->isActualEnding = TRUE;
			break;
		case ENDING_THREE:
			scene_init
			(
				&_e->sceneHandler, 
				_e->g, 
				(Scene*)ENDING_THREE_SCENES,
				ENDING_THREE_SCENE_COUNT
			);
			_e->isActualEnding = TRUE;
			break;
		default:
			break;
	}

	text_sprite_init
	(
		&plusButtonSprite,
		_e->g->renderer,
		_e->g->fonts[FONT_BIG],
		ENDING_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_ENDING_NEW_GAME_PLUS,
		0
	);

	plusButtonSprite.x = ENDING_PLUS_BUTTON_OFFSET_X;
	plusButtonSprite.y = ENDING_PLUS_BUTTON_OFFSET_Y;

	text_sprite_init
	(
		&endButtonSprite,
		_e->g->renderer,
		_e->g->fonts[FONT_BIG],
		ENDING_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_ENDING_RESULTS,
		0
	);

	endButtonSprite.x = ENDING_END_BUTTON_OFFSET_X;
	endButtonSprite.y = ENDING_END_BUTTON_OFFSET_Y;

	button_init(&_e->plusButton, &plusButtonSprite);
	button_init(&_e->endButton, &endButtonSprite);

	cursor_init(&_e->cursor, &_e->g->mouse, _e->g->textures[TEXTURE_GUI_CURSOR]);

	music_play(_e->g->music[MUSIC_WIND], TRUE);
}

/* Frees ending. */
void
ending_free(Ending* _e)
{
	if (_e->plusButton.sprite.atlas.texture)
		texture_free(_e->plusButton.sprite.atlas.texture);

	if (_e->endButton.sprite.atlas.texture)
		texture_free(_e->endButton.sprite.atlas.texture);

	scene_free(&_e->sceneHandler, _e->g);
}

/* Updates ending. */
void
ending_update(Ending* _e)
{
	scene_update(&_e->sceneHandler, _e->g);

	cursor_update(&_e->cursor, &_e->g->mouse);

	if (_e->sceneHandler.isEnd)
	{
		if (_e->isActualEnding)
		{
			button_update(&_e->plusButton, &_e->g->mouse);
			button_update(&_e->endButton, &_e->g->mouse);

			if (_e->plusButton.state == BUTTON_STATE_PRESS)
			{
				_e->isNewGamePlus = TRUE;
				sound_play(_e->g->sounds[SOUND_MENU], -1);
			}
			else if (_e->endButton.state == BUTTON_STATE_PRESS)
			{
				_e->isResults = TRUE;
				sound_play(_e->g->sounds[SOUND_MENU], -1);
			}
		}
		else
		{
			if (mouse_press(&_e->g->mouse, MOUSE_LEFT))
			{
				_e->isResults = TRUE;
				sound_play(_e->g->sounds[SOUND_MENU], -1);
			}
		}
	}
}

/* Draws ending. */
void
ending_draw(Ending* _e)
{
	scene_draw(&_e->sceneHandler, _e->g);

	cursor_draw(&_e->cursor, _e->g->renderer);

	if (_e->isActualEnding && _e->sceneHandler.isEnd)
	{
		button_draw(&_e->plusButton, _e->g->renderer);
		button_draw(&_e->endButton, _e->g->renderer);
	}
}
