/*
 * DESCRIPTION:
 * Main game functions.
*/

#include "game.h"

static void game_update(Game* _g);
static void game_draw(Game* _g);

/* Updates the game. */
static void
game_update(Game* _g)
{
        event_update(&_g->event);
	mouse_update(&_g->mouse);
	keyboard_update(&_g->keyboard);

	game_state_update(_g);

	/*
	if (keyboard_press(&_g->keyboard, KEYBOARD_F))
	{
		if (_g->isFullScreen)
			window_fullscreen_enter(_g->window);
		else
			window_fullscreen_exit(_g->window);


		_g->isFullScreen = !_g->isFullScreen;
	}
	*/
}

/* Draws the game. */
static void
game_draw(Game* _g)
{
        renderer_color_set(_g->renderer, RENDERER_COLOR);
        renderer_clear(_g->renderer);

	game_state_draw(_g);

        renderer_present(_g->renderer);
}

/* Initializes game. */
void
game_init(Game* _g)
{
        memset(_g, '\0', sizeof(Game));

        window_init
        (
                &_g->window,
                WINDOW_TITLE,
                WINDOW_WIDTH,
                WINDOW_HEIGHT,
                WINDOW_FLAGS
        );

        renderer_init
        (
                &_g->renderer,
                _g->window,
                RENDERER_FLAGS
        );

	SDL_ShowCursor(SDL_DISABLE);

	game_state_transition(_g, STATE_TITLE, TRUE, TRUE, TRUE, TRUE);

	_g->stateTimer = STATE_TRANSITION_TIMER_MAX;
}

/* Frees game resources. */
void
game_free(Game* _g)
{
	game_state_free(_g);
	game_music_clear(_g);
	game_font_clear(_g);
	game_sound_clear(_g);
	game_texture_clear(_g);
}

/* Main game loop. */
void
game_loop(Game* _g)
{
        tick_update(&_g->tick);

        while (_g->tick.cum > FRAME_DELAY)
        {
                game_update(_g);
                _g->tick.cum -= FRAME_DELAY;
        }

        game_draw(_g);
}
