/*
 * DESCRIPTION:
 * Basic GUI button.
*/

#include "button.h"

/* Initializes a button. */
void
button_init(Button* _b, Sprite* _s)
{
	memset(_b, '\0', sizeof(Button));

	_b->sprite = *_s;
}

/* Updates a button. */
void
button_update(Button* _b, Mouse* _m)
{
	SDL_Rect rect;

	rect = sprite_rect_get(&_b->sprite);

	_b->state = BUTTON_STATE_NONE;

	if (rectangle_point_collide(&rect, _m->x, _m->y))
	{
		if (_b->state == BUTTON_STATE_HOVER)
			_b->state = BUTTON_STATE_HOVERING;
		else
			_b->state = BUTTON_STATE_HOVER;

		if (mouse_held(_m, MOUSE_LEFT))
			_b->state = BUTTON_STATE_HELD;
		else if (mouse_press(_m, MOUSE_LEFT))
			_b->state = BUTTON_STATE_PRESS;

	}
	else
		_b->state = BUTTON_STATE_NONE;

	switch (_b->state)
	{
		case BUTTON_STATE_PRESS:
			_b->sprite.color = BUTTON_PRESS_COLOR;
			break;
		case BUTTON_STATE_HELD:
			_b->sprite.color = BUTTON_HELD_COLOR;
			break;
		case BUTTON_STATE_HOVER:
		case BUTTON_STATE_HOVERING:
			_b->sprite.color = BUTTON_HOVER_COLOR;
			break;
		case BUTTON_STATE_NONE:
		default:
			_b->sprite.color = BUTTON_NONE_COLOR;
			break;
	}
}

/* Draws a button. */
void
button_draw(Button* _b, SDL_Renderer* _r)
{
	sprite_draw(&_b->sprite, _r);
}
