/*
 * DESCRIPTION:
 * Gameplay's cursor.
*/

#include "cursor.h"

/* Initializes the cursor. */
void
cursor_init(Cursor* _c, Mouse* _m, SDL_Texture* _t)
{
	Atlas atlas;

	atlas_init
	(
		&atlas,
		_t,
		CURSOR_ROWS,
		CURSOR_COLUMNS,
		CURSOR_FRAME_WIDTH,
		CURSOR_FRAME_HEIGHT
	);

	sprite_atlas_init
	(
		&_c->sprite,
		&atlas,
		ORIGIN_MIDDLE,
		CURSOR_WIDTH,
		CURSOR_HEIGHT
	);

	_c->sprite.frame = 0;
	_c->sprite.x = _m->x;
	_c->sprite.y = _m->y;

	_c->sprite.color.a = CURSOR_ALPHA;
}

/* Frees cursor resources. */
void
cursor_free(Cursor* _c)
{

}

/* Updates the cursor. */
void
cursor_update(Cursor* _c, Mouse* _m)
{
	if (mouse_held(_m, MOUSE_LEFT))
		_c->sprite.frame = 1;
	else if (mouse_held(_m, MOUSE_RIGHT))
		_c->sprite.frame = 2;
	else
		_c->sprite.frame = 0;

	_c->sprite.x = _m->x;
	_c->sprite.y = _m->y;

	_c->sprite.x = _c->sprite.x < CURSOR_X_MIN ? CURSOR_X_MIN : _c->sprite.x;
	_c->sprite.x = _c->sprite.x > CURSOR_X_MAX ? CURSOR_X_MAX : _c->sprite.x;
	_c->sprite.y = _c->sprite.y < CURSOR_Y_MIN ? CURSOR_Y_MIN : _c->sprite.y;
	_c->sprite.y = _c->sprite.y > CURSOR_Y_MAX ? CURSOR_Y_MAX : _c->sprite.y;
}

/* Draws the cursor. */
void
cursor_draw(Cursor* _c, SDL_Renderer* _r)
{
	sprite_draw(&_c->sprite, _r);

}
