/*
 * DESCRIPTION:
 * Displays game info (credits/licenses).
*/

#include "info.h"

/* Initialize info. */
void
info_init(Info* _i, Game* _g)
{
	Sprite buttonSprite;

	memset(_i, '\0', sizeof(Info));

	_i->g = _g;

	cursor_init(&_i->cursor, &_i->g->mouse, _i->g->textures[TEXTURE_GUI_CURSOR]);

	text_sprite_init
	(
		&_i->licenseText,
		_i->g->renderer,
		_i->g->fonts[FONT_SMALL],
		INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_INFO_LICENSES,
		INFO_LICENSE_TEXT_WRAP
	);

	_i->licenseText.x = INFO_LICENSE_TEXT_OFFSET_X;
	_i->licenseText.y = INFO_LICENSE_TEXT_OFFSET_Y;

	sprite_init
	(
		&_i->themeSprite,
		_i->g->textures[TEXTURE_INFO_THEME],
		ORIGIN_MIDDLE,
		INFO_THEME_WIDTH,
		INFO_THEME_HEIGHT
	);

	_i->themeSprite.x = INFO_THEME_OFFSET_X;
	_i->themeSprite.y = INFO_THEME_OFFSET_Y;

	sprite_init
	(
		&_i->wildcardAdminSprite,
		_i->g->textures[TEXTURE_INFO_WILDCARD_ADMIN],
		ORIGIN_MIDDLE,
		INFO_WILDCARD_WIDTH,
		INFO_WILDCARD_HEIGHT
	);

	_i->wildcardAdminSprite.x = INFO_WILDCARD_ADMIN_OFFSET_X;
	_i->wildcardAdminSprite.y = INFO_WILDCARD_ADMIN_OFFSET_Y;

	sprite_init
	(
		&_i->wildcardPatreonSprite,
		_i->g->textures[TEXTURE_INFO_WILDCARD_PATREON],
		ORIGIN_MIDDLE,
		INFO_WILDCARD_WIDTH,
		INFO_WILDCARD_HEIGHT
	);

	_i->wildcardPatreonSprite.x = INFO_WILDCARD_PATREON_OFFSET_X;
	_i->wildcardPatreonSprite.y = INFO_WILDCARD_PATREON_OFFSET_Y;

	sprite_init
	(
		&_i->wildcardCommunitySprite,
		_i->g->textures[TEXTURE_INFO_WILDCARD_COMMUNITY],
		ORIGIN_MIDDLE,
		INFO_WILDCARD_WIDTH,
		INFO_WILDCARD_HEIGHT
	);

	_i->wildcardCommunitySprite.x = INFO_WILDCARD_COMMUNITY_OFFSET_X;
	_i->wildcardCommunitySprite.y = INFO_WILDCARD_COMMUNITY_OFFSET_Y;

	sprite_init
	(
		&_i->backBackgroundSprite,
		_i->g->textures[TEXTURE_GUI_SIGN_SMALL],
		ORIGIN_MIDDLE,
		INFO_BUTTON_BACKGROUND_WIDTH,
		INFO_BUTTON_BACKGROUND_HEIGHT
	);

	_i->backBackgroundSprite.x = INFO_BUTTON_OFFSET_X;
	_i->backBackgroundSprite.y = INFO_BUTTON_OFFSET_Y;

	text_sprite_init
	(
		&buttonSprite,
		_i->g->renderer,
		_i->g->fonts[FONT_MEDIUM],
		INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_INFO_BACK,
		0
	);

	buttonSprite.rotation = INFO_BUTTON_TEXT_ROTATION;

	buttonSprite.x = _i->backBackgroundSprite.x;
	buttonSprite.y = _i->backBackgroundSprite.y;

	button_init(&_i->backButton, &buttonSprite);

	text_sprite_init
	(
		&_i->themeText,
		_i->g->renderer,
		_i->g->fonts[FONT_TINY],
		INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_INFO_THEME,
		INFO_THEME_WRAP
	);

	_i->themeText.x = INFO_THEME_TEXT_OFFSET_X;
	_i->themeText.y = INFO_THEME_TEXT_OFFSET_Y;

	text_sprite_init
	(
		&_i->wildcardAdminText,
		_i->g->renderer,
		_i->g->fonts[FONT_TINY],
		INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_INFO_WILDCARD_ADMIN,
		INFO_WILDCARD_WRAP
	);

	_i->wildcardAdminText.x = INFO_WILDCARD_ADMIN_TEXT_OFFSET_X;
	_i->wildcardAdminText.y = INFO_WILDCARD_ADMIN_TEXT_OFFSET_Y;

	text_sprite_init
	(
		&_i->wildcardPatreonText,
		_i->g->renderer,
		_i->g->fonts[FONT_TINY],
		INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_INFO_WILDCARD_PATREON,
		INFO_WILDCARD_WRAP
	);

	_i->wildcardPatreonText.x = INFO_WILDCARD_PATREON_TEXT_OFFSET_X;
	_i->wildcardPatreonText.y = INFO_WILDCARD_PATREON_TEXT_OFFSET_Y;

	text_sprite_init
	(
		&_i->wildcardCommunityText,
		_i->g->renderer,
		_i->g->fonts[FONT_TINY],
		INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_INFO_WILDCARD_COMMUNITY,
		INFO_WILDCARD_WRAP
	);

	_i->wildcardCommunityText.x = INFO_WILDCARD_COMMUNITY_TEXT_OFFSET_X;
	_i->wildcardCommunityText.y = INFO_WILDCARD_COMMUNITY_TEXT_OFFSET_Y;
}

/* Frees info. */
void
info_free(Info* _i)
{
	if (_i->licenseText.atlas.texture)
		texture_free(_i->licenseText.atlas.texture);

	if (_i->backButton.sprite.atlas.texture)
		texture_free(_i->backButton.sprite.atlas.texture);

	if (_i->themeText.atlas.texture)
		texture_free(_i->themeText.atlas.texture);

	if (_i->wildcardAdminText.atlas.texture)
		texture_free(_i->wildcardAdminText.atlas.texture);

	if (_i->wildcardPatreonText.atlas.texture)
		texture_free(_i->wildcardPatreonText.atlas.texture);

	if (_i->wildcardCommunityText.atlas.texture)
		texture_free(_i->wildcardCommunityText.atlas.texture);

	cursor_free(&_i->cursor);
}

/* Updates info. */
void
info_update(Info* _i)
{
	cursor_update(&_i->cursor, &_i->g->mouse);

	button_update(&_i->backButton, &_i->g->mouse);

	if (_i->backButton.state == BUTTON_STATE_PRESS)
	{
		_i->isBackPressed = TRUE;

		sound_play(_i->g->sounds[SOUND_MENU], -1);
	}
}

/* Draw info. */
void
info_draw(Info* _i)
{
	SDL_Rect r;

	r.x = 0;
	r.y = 0;
	r.w = INFO_RECTANGLE_WIDTH;
	r.h = INFO_RECTANGLE_HEIGHT;

	rectangle_draw(&r, _i->g->renderer, INFO_BACKGROUND_COLOR);

	sprite_draw(&_i->licenseText, _i->g->renderer);

	sprite_draw(&_i->themeSprite, _i->g->renderer);
	sprite_draw(&_i->wildcardAdminSprite, _i->g->renderer);
	sprite_draw(&_i->wildcardPatreonSprite, _i->g->renderer);
	sprite_draw(&_i->wildcardCommunitySprite, _i->g->renderer);

	sprite_draw(&_i->themeText, _i->g->renderer);
	sprite_draw(&_i->wildcardAdminText, _i->g->renderer);
	sprite_draw(&_i->wildcardPatreonText, _i->g->renderer);
	sprite_draw(&_i->wildcardCommunityText, _i->g->renderer);

	sprite_draw(&_i->backBackgroundSprite, _i->g->renderer);
	button_draw(&_i->backButton, _i->g->renderer);

	cursor_draw(&_i->cursor, _i->g->renderer);
}
