/*
 * DESCRIPTION:
 * Collectible code.
*/

#include "collectible.h"

/* Adds a new collectible. */
Collectible*
collectible_add(List* _l, CollectibleType _c, f64 _x, f64 _y, Play* _p)
{
	Collectible collectible;
	f64 angle;

	memset(&collectible, '\0', sizeof(Collectible));

	collectible.type = _c;

	switch (collectible.type)
	{
		case COLLECTIBLE_DOLLARS:
			dollars_init(&collectible, _l, _p);
			break;
		case COLLECTIBLE_SACK:
			sack_init(&collectible, _l, _p);
			break;
		case COLLECTIBLE_BULLIONS:
			bullions_init(&collectible, _l, _p);
			break;
		case COLLECTIBLE_MIRACLE_CURE:
			miracle_cure_init(&collectible, _l, _p);
			break;
		case COLLECTIBLE_DYNAMITE:
			dynamite_init(&collectible, _l, _p);
			break;
		case COLLECTIBLE_BERSERK:
			berserk_init(&collectible, _l, _p);
			break;
		default:
			break;
	}


	collectible.sprite.x = _x;
	collectible.sprite.y = _y;

	collectible.shadow.x = collectible.sprite.x + COLLECTIBLE_SHADOW_OFFSET_Y;
	collectible.shadow.y = collectible.sprite.y + COLLECTIBLE_SHADOW_OFFSET_Y;

	collectible.shadow.r = COLLECTIBLE_SHADOW_RADIUS;

	collectible.lifetime = COLLECTIBLE_LIFETIME;

	collectible.isActive = FALSE;
	collectible.activeTimer = COLLECTIBLE_ACTIVE_TIMER_MAX;

	angle = random_f64_in_range_get(0, ROTATION_MAX_RAD);

	collectible.dX = cos(angle) * COLLECTIBLE_VELOCITY;
	collectible.dY = sin(angle) * COLLECTIBLE_VELOCITY;

	sound_play(_p->g->sounds[SOUND_COLLECTIBLE], -1);

	return (Collectible*)list_append_new(_l, (void*)&collectible, sizeof(Collectible));
}

/* Removes a collectible. */
void
collectible_remove(Collectible* _c, List* _l, Play* _p)
{
	list_node_from_data_remove(_l, _c, sizeof(Collectible));
}

/* A collectible is collected. */
void
collectible_collect(Collectible* _c, List* _l, Play* _p)
{
	switch (_c->type)
	{
		case COLLECTIBLE_DOLLARS:
			dollars_collect(_c, _l, _p);
			break;
		case COLLECTIBLE_SACK:
			sack_collect(_c, _l, _p);
			break;
		case COLLECTIBLE_BULLIONS:
			bullions_collect(_c, _l, _p);
			break;
		case COLLECTIBLE_MIRACLE_CURE:
			miracle_cure_collect(_c, _l, _p);
			break;
		case COLLECTIBLE_DYNAMITE:
			dynamite_collect(_c, _l, _p);
			break;
		case COLLECTIBLE_BERSERK:
			berserk_collect(_c, _l, _p);
			break;
		default:
			break;
	}

	collectible_remove(_c, _l, _p);
}

/* Updates a collectible. */
void
collectible_update(Collectible* _c, List* _l, Play* _p)
{
	_c->sprite.x += _c->dX;
	_c->sprite.y += _c->dY;

	_c->shadow.x = _c->sprite.x + COLLECTIBLE_SHADOW_OFFSET_X;
	_c->shadow.y = _c->sprite.y + COLLECTIBLE_SHADOW_OFFSET_Y;

	_c->dX *= COLLECTIBLE_FRICTION;
	_c->dY *= COLLECTIBLE_FRICTION;

	_c->lifetime--;

	if (!_c->isActive)
	{
		f64 progress;

		_c->activeTimer--;

		progress = (f64)_c->activeTimer / COLLECTIBLE_ACTIVE_TIMER_MAX;

		_c->sprite.color.a = SDL_ALPHA_OPAQUE * (1 - progress);

		if (_c->activeTimer <= 0)
		{
			_c->sprite.color.a = SDL_ALPHA_OPAQUE;
			_c->activeTimer = 0;
			_c->isActive = TRUE;
		}
	}

	if (_c->attachedSnake)
        {
                _c->sprite.x = _c->attachedSnake->sprite.x;
                _c->sprite.y = _c->attachedSnake->sprite.y;
        }

	if (circle_collide(&_c->shadow, &_p->player.shadow))
		collectible_collect(_c, _l, _p);
	else
	{
		if (_c->lifetime <= 0)
			collectible_remove(_c, _l, _p);
		else if (_c->lifetime <= COLLECTIBLE_DISAPPEAR_THRESHOLD)
		{
			if (_c->lifetime % COLLECTIBLE_DISAPPEAR_MODULO == 0)
				_c->sprite.color.a = 0;
			else
				_c->sprite.color.a = SDL_ALPHA_OPAQUE;
		}
	}
}
