/*
 * DESCRIPTION:
 * AI for the "Chip" enemy.
*/

#include "chip.h"

static s32 chip_health_get(Enemy* _e, List* _l, Play* _p);
static f64 chip_velocity_get(Enemy* _e, List* _l, Play* _p);

/* Gets the health of the Chip. */
static s32
chip_health_get(Enemy* _e, List* _l, Play* _p)
{
	s32 health;

	health = CHIP_HEALTH_BASE + ((_p->game - 1) * CHIP_HEALTH_GAME_MULTIPLIER);

	health = CEIL(health, CHIP_HEALTH_MAX);

	return health;
}

/* Gets the velocity of the Chip. */
static f64
chip_velocity_get(Enemy* _e, List* _l, Play* _p)
{
	f64 velocity;

	velocity = CHIP_VELOCITY_BASE + ((_p->game - 1) * CHIP_VELOCITY_GAME_MULTIPLIER);

	velocity = CEIL(velocity, CHIP_VELOCITY_MAX);

	return velocity;
}

/* Initializes Chip. */
void
chip_init(Enemy* _e, List* _l, Play* _p)
{
	sprite_init
	(
		&_e->sprite,
		_p->g->textures[TEXTURE_ENEMY_CHIP],
		ORIGIN_MIDDLE,
		CHIP_WIDTH,
		CHIP_HEIGHT
	);

	_e->shadow.r = CHIP_SHADOW_RADIUS;
}

/* Sets the Chip. */
void
chip_set(Enemy* _e, List* _l, f64 _x, f64 _y, Play* _p)
{
	f64 angle;
	f64 velocity;

	_e->sprite.x = _x;
	_e->sprite.y = _y;

	_e->shadow.x = _e->sprite.x + CHIP_SHADOW_OFFSET_X;
	_e->shadow.y = _e->sprite.y + CHIP_SHADOW_OFFSET_Y;

	_e->sprite.rotation = random_f64_in_range_get(0, ROTATION_MAX_DEG);

	velocity = chip_velocity_get(_e, _l, _p);

	angle = math_atan
	(
		_e->shadow.x,
		_e->shadow.y,
		_p->player.shadow.x,
		_p->player.shadow.y
	);

	_e->dX = -cos(angle) * velocity;
	_e->dY = -sin(angle) * velocity;

	_e->health = chip_health_get(_e, _l, _p);
	_e->isActive = TRUE;
	_e->lifetime = CHIP_LIFETIME;
}

/* 
 * Updates Chip.
 * Just flies towards the player, and will remove
 * itself if enough time has passed.
*/
void
chip_update(Enemy* _e, List* _l, Play* _p)
{
	SDL_Rect rect;
	SDL_Rect windowRect;

	rect = sprite_rect_get(&_e->sprite);
	windowRect = window_rect_get(_p->g->window);

	_e->lifetime--;

	_e->sprite.x += _e->dX;
	_e->sprite.y += _e->dY;

	_e->sprite.rotation += CHIP_ROTATION_DELTA;

	_e->shadow.x = _e->sprite.x + CHIP_SHADOW_OFFSET_X;
	_e->shadow.y = _e->sprite.y + CHIP_SHADOW_OFFSET_Y;

	if 
	(
		_e->lifetime <= 0 ||
		(_e->lifetime < CHIP_LIFETIME / 2 && !SDL_HasIntersection(&rect, &windowRect))
	)
		enemy_remove(_l, _e, _p);
}
