/*
 * DESCRIPTION:
 * Displays a bounty icon and the current bountys available.
*/

#include "gui_bounty.h"

/* Sets the gui bounty text. */
static void 
gui_bounty_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_BOUNTY_TEXT_MAX);

	if (_g->bountyText.atlas.texture)
		texture_free(_g->bountyText.atlas.texture);

	if (_p->player.isBeingDefeated)
	{
		snprintf
		(
			s,
			GUI_BOUNTY_TEXT_MAX,
			"%s",
			STRING_PLAY_GUI_BOUNTY_DEAD
		);
	}
	else if (_p->isBetweenWaves)
	{
		snprintf
		(
			s,
			GUI_BOUNTY_TEXT_MAX,
			"%s",
			STRING_PLAY_GUI_BOUNTY_NONE
		);
	}
	else
	{
		snprintf
		(
			s,
			GUI_BOUNTY_TEXT_MAX,
			"%i",
			_p->enemyMax
		);
	}

	text_sprite_init
	(
		&_g->bountyText,
		_p->g->renderer,
		_p->g->fonts[FONT_BIG],
		GUI_BOUNTY_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	_g->bountyText.x = _g->bountyBackgroundSprite.x;
	_g->bountyText.y = _g->bountyBackgroundSprite.y;

	_g->bountyText.rotation = GUI_BOUNTY_TEXT_ROTATION;
	_g->bountyText.color.a = GUI_BOUNTY_ALPHA;

	free(s);
}

/* Initializes bounty GUI. */
void 
gui_bounty_init(GUI* _g, Play* _p)
{
	sprite_init
	(
		&_g->bountyBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_SMALL],
		ORIGIN_MIDDLE,
		GUI_BOUNTY_BACKGROUND_WIDTH,
		GUI_BOUNTY_BACKGROUND_HEIGHT
	);

	_g->bountyBackgroundSprite.x = GUI_BOUNTY_SPRITE_OFFSET_X;
	_g->bountyBackgroundSprite.y = GUI_BOUNTY_SPRITE_OFFSET_Y;

	_g->bountyBackgroundSprite.color.a = GUI_BOUNTY_ALPHA;

	gui_bounty_text_set(_g, _p);
}

/* Frees bounty GUI. */
void 
gui_bounty_free(GUI* _g, Play* _p)
{
	if (_g->bountyText.atlas.texture)
		texture_free(_g->bountyText.atlas.texture);
}

/* Updates bounty GUI. */
void 
gui_bounty_update(GUI* _g, Play* _p)
{
	if (_p->isBountyUpdate)
	{
		gui_bounty_text_set(_g, _p);
		_p->isBountyUpdate = FALSE;
	}
}

/* Draws bounty GUI. */
void 
gui_bounty_draw(GUI* _g, Play* _p)
{
	sprite_draw(&_g->bountyBackgroundSprite, _p->g->renderer);
	sprite_draw(&_g->bountyText, _p->g->renderer);
}
