/*
 * DESCRIPTION:
 * Displays a cash icon and the current cashs available.
*/

#include "gui_cash.h"

/* Sets the gui cash text. */
static void 
gui_cash_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_CASH_TEXT_MAX);

	if (_g->cashText.atlas.texture)
		texture_free(_g->cashText.atlas.texture);

	snprintf
	(
		s,
		GUI_CASH_TEXT_MAX,
		"%09i",
		_p->player.cash
	);

	text_sprite_init
	(
		&_g->cashText,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		GUI_CASH_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	_g->cashText.x = _g->cashTextBackgroundSprite.x;
	_g->cashText.y = _g->cashTextBackgroundSprite.y;

	_g->cashText.rotation = GUI_CASH_TEXT_ROTATION;
	_g->cashText.color.a = GUI_CASH_ALPHA;

	free(s);
}

/* Initializes cash GUI. */
void 
gui_cash_init(GUI* _g, Play* _p)
{
	sprite_init
	(
		&_g->cashSprite,
		_p->g->textures[TEXTURE_GUI_CASH],
		ORIGIN_MIDDLE,
		GUI_CASH_WIDTH,
		GUI_CASH_HEIGHT
	);

	sprite_init
	(
		&_g->cashBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_SMALL],
		ORIGIN_MIDDLE,
		GUI_CASH_BACKGROUND_WIDTH,
		GUI_CASH_BACKGROUND_HEIGHT
	);

	sprite_init
	(
		&_g->cashTextBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_BIG],
		ORIGIN_MIDDLE,
		GUI_CASH_TEXT_BACKGROUND_WIDTH,
		GUI_CASH_TEXT_BACKGROUND_HEIGHT
	);
	
	_g->cashBackgroundSprite.x = GUI_CASH_SPRITE_OFFSET_X;
	_g->cashBackgroundSprite.y = GUI_CASH_SPRITE_OFFSET_Y;

	_g->cashTextBackgroundSprite.x = GUI_CASH_TEXT_OFFSET_X;
	_g->cashTextBackgroundSprite.y = GUI_CASH_TEXT_OFFSET_Y;

	_g->cashSprite.x = _g->cashBackgroundSprite.x;
	_g->cashSprite.y = _g->cashBackgroundSprite.y;

	_g->cashBackgroundSprite.color.a = GUI_CASH_ALPHA;
	_g->cashTextBackgroundSprite.color.a = GUI_CASH_ALPHA;
	_g->cashSprite.color.a = GUI_CASH_ALPHA;

	gui_cash_text_set(_g, _p);
}

/* Frees cash GUI. */
void 
gui_cash_free(GUI* _g, Play* _p)
{
	if (_g->cashText.atlas.texture)
		texture_free(_g->cashText.atlas.texture);
}

/* Updates cash GUI. */
void 
gui_cash_update(GUI* _g, Play* _p)
{
	if (_p->player.isCashUpdate)
		gui_cash_text_set(_g, _p);
}

/* Draws cash GUI. */
void 
gui_cash_draw(GUI* _g, Play* _p)
{
	sprite_draw(&_g->cashBackgroundSprite, _p->g->renderer);
	sprite_draw(&_g->cashTextBackgroundSprite, _p->g->renderer);
	sprite_draw(&_g->cashSprite, _p->g->renderer);
	sprite_draw(&_g->cashText, _p->g->renderer);
}
