/*
 * DESCRIPTION:
 * Displays a redirect icon and the current redirects available.
*/

#include "gui_redirect.h"

/* Sets the gui redirect text. */
static void 
gui_redirect_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_REDIRECT_TEXT_MAX);

	snprintf
	(
		s,
		GUI_REDIRECT_TEXT_MAX,
		"%i/%i",
		_p->player.redirects,
		_p->player.redirectMax
	);

	if (_g->redirectText.atlas.texture)
		texture_free(_g->redirectText.atlas.texture);

	text_sprite_init
	(
		&_g->redirectText,
		_p->g->renderer,
		_p->g->fonts[FONT_SMALL],
		GUI_REDIRECT_TEXT_COLOR,
		ORIGIN_TOP_LEFT,
		s,
		0
	);

	free(s);

	_g->redirectText.x = _p->gui.cursor.sprite.x + GUI_REDIRECT_TEXT_OFFSET_X;
	_g->redirectText.y = _p->gui.cursor.sprite.y + GUI_REDIRECT_TEXT_OFFSET_Y;
}

/* Initializes redirect GUI. */
void 
gui_redirect_init(GUI* _g, Play* _p)
{
	sprite_init
	(
		&_g->redirectSprite,
		_p->g->textures[TEXTURE_GUI_REDIRECT],
		ORIGIN_MIDDLE,
		GUI_REDIRECT_WIDTH,
		GUI_REDIRECT_HEIGHT
	);

	_g->redirectSprite.color.a = GUI_REDIRECT_ALPHA;

	_g->redirectSprite.x = _p->gui.cursor.sprite.x + GUI_REDIRECT_SPRITE_OFFSET_X;
	_g->redirectSprite.y = _p->gui.cursor.sprite.y + GUI_REDIRECT_SPRITE_OFFSET_Y;

	gui_redirect_text_set(_g, _p);
}

/* Frees redirect GUI. */
void 
gui_redirect_free(GUI* _g, Play* _p)
{
	if (_g->redirectText.atlas.texture)
		texture_free(_g->redirectText.atlas.texture);
}

/* Updates redirect GUI. */
void 
gui_redirect_update(GUI* _g, Play* _p)
{
	_g->redirectSprite.x = _p->gui.cursor.sprite.x + GUI_REDIRECT_SPRITE_OFFSET_X;
	_g->redirectSprite.y = _p->gui.cursor.sprite.y + GUI_REDIRECT_SPRITE_OFFSET_Y;

	_g->redirectText.x = _p->gui.cursor.sprite.x + GUI_REDIRECT_TEXT_OFFSET_X;
	_g->redirectText.y = _p->gui.cursor.sprite.y + GUI_REDIRECT_TEXT_OFFSET_Y;

	if (_p->player.isRedirectUpdate)
		gui_redirect_text_set(_g, _p);
}

/* Draws redirect GUI. */
void 
gui_redirect_draw(GUI* _g, Play* _p)
{
	sprite_draw(&_g->redirectSprite, _p->g->renderer);
	sprite_draw(&_g->redirectText, _p->g->renderer);
}
