/*
 * DESCRIPTION:
 * Displays a snake icon and the current snakes available.
*/

#include "gui_snake.h"

/* Sets the gui snake text. */
static void 
gui_snake_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_SNAKE_TEXT_MAX);

	if (_g->snakeText.atlas.texture)
		texture_free(_g->snakeText.atlas.texture);

	if (_p->player.isPowerUp)
	{
		snprintf
		(
			s,
			GUI_SNAKE_TEXT_MAX,
			"%s",
			STRING_PLAY_GUI_SNAKE_POWER_UP
		);
	}
	else
	{
		snprintf
		(
			s,
			GUI_SNAKE_TEXT_MAX,
			"%i/%i",
			_p->player.snakeReserve,
			_p->player.snakeMax
		);
	}

	text_sprite_init
	(
		&_g->snakeText,
		_p->g->renderer,
		_p->g->fonts[FONT_SMALL],
		GUI_SNAKE_TEXT_COLOR,
		ORIGIN_TOP_LEFT,
		s,
		0
	);

	free(s);

	_g->snakeText.x = _p->gui.cursor.sprite.x + GUI_SNAKE_TEXT_OFFSET_X;
	_g->snakeText.y = _p->gui.cursor.sprite.y + GUI_SNAKE_TEXT_OFFSET_Y;
}

/* Initializes snake GUI. */
void 
gui_snake_init(GUI* _g, Play* _p)
{
	sprite_init
	(
		&_g->snakeSprite,
		_p->g->textures[TEXTURE_GUI_SNAKE],
		ORIGIN_MIDDLE,
		GUI_SNAKE_WIDTH,
		GUI_SNAKE_HEIGHT
	);

	_g->snakeSprite.color.a = GUI_SNAKE_ALPHA;

	_g->snakeSprite.x = _p->gui.cursor.sprite.x + GUI_SNAKE_SPRITE_OFFSET_X;
	_g->snakeSprite.y = _p->gui.cursor.sprite.y + GUI_SNAKE_SPRITE_OFFSET_Y;

	gui_snake_text_set(_g, _p);
}

/* Frees snake GUI. */
void 
gui_snake_free(GUI* _g, Play* _p)
{
	if (_g->snakeText.atlas.texture)
		texture_free(_g->snakeText.atlas.texture);
}

/* Updates snake GUI. */
void 
gui_snake_update(GUI* _g, Play* _p)
{
	_g->snakeSprite.x = _p->gui.cursor.sprite.x + GUI_SNAKE_SPRITE_OFFSET_X;
	_g->snakeSprite.y = _p->gui.cursor.sprite.y + GUI_SNAKE_SPRITE_OFFSET_Y;

	_g->snakeText.x = _p->gui.cursor.sprite.x + GUI_SNAKE_TEXT_OFFSET_X;
	_g->snakeText.y = _p->gui.cursor.sprite.y + GUI_SNAKE_TEXT_OFFSET_Y;

	if (_p->player.isSnakeUpdate)
		gui_snake_text_set(_g, _p);
}

/* Draws snake GUI. */
void 
gui_snake_draw(GUI* _g, Play* _p)
{
	sprite_draw(&_g->snakeSprite, _p->g->renderer);
	sprite_draw(&_g->snakeText, _p->g->renderer);
}
