/*
 * DESCRIPTION:
 * Upgrade display.
*/

#include "gui_upgrade.h"

static void gui_upgrade_no_cash(GUI* _g, Play* _p);
static void gui_upgrade_maxed(GUI* _g, Play* _p);
static void gui_upgrade_snake(GUI* _g, Play* _p);
static void gui_upgrade_lifetime(GUI* _g, Play* _p);
static void gui_upgrade_redirect(GUI* _g, Play* _p);
static void gui_upgrade_snake_text_set(GUI* _g, Play* _p);


/* No cash!! */
static void
gui_upgrade_no_cash(GUI* _g, Play* _p)
{
	message_no_cash_add(_p->player.sprite.x, _p->player.sprite.y, _p);

	sound_play(_p->g->sounds[SOUND_ERROR], -1);
}

/* Maxed out!! */
static void
gui_upgrade_maxed(GUI* _g, Play* _p)
{
	message_maxed_add(_p->player.sprite.x, _p->player.sprite.y, _p);

	sound_play(_p->g->sounds[SOUND_ERROR], -1);
}

/* Upgrade snake level. */
static void
gui_upgrade_snake(GUI* _g, Play* _p)
{
	if (_p->player.cash >= _p->player.snakeUpgradePrice)
	{
		_p->player.cash -= _p->player.snakeUpgradePrice;

		message_purchase_add(_p->player.sprite.x, _p->player.sprite.y, _p->player.snakeUpgradePrice, _p);

		_p->player.snakeUpgradeLevel++;

		if (_p->player.snakeUpgradeLevel > GUI_UPGRADE_MAX)
		{
			_p->player.snakeUpgradeLevel = CEIL(_p->player.snakeUpgradeLevel, GUI_UPGRADE_MAX);
			gui_upgrade_maxed(_g, _p);
		}
		else
		{
			_p->player.isSnakeUpgradeUpdate = TRUE;

			_p->player.isSnakeUpdate = TRUE;

			sound_play(_p->g->sounds[SOUND_PURCHASE], -1);
		}
	}
	else
		gui_upgrade_no_cash(_g, _p);
}

/* Upgrade snake level. */
static void
gui_upgrade_lifetime(GUI* _g, Play* _p)
{
	if (_p->player.cash >= _p->player.lifetimeUpgradePrice)
	{
		_p->player.cash -= _p->player.lifetimeUpgradePrice;

		message_purchase_add(_p->player.sprite.x, _p->player.sprite.y, _p->player.lifetimeUpgradePrice, _p);

		_p->player.lifetimeUpgradeLevel++;

		if (_p->player.snakeUpgradeLevel > GUI_UPGRADE_MAX)
			_p->player.lifetimeUpgradeLevel = CEIL(_p->player.lifetimeUpgradeLevel, GUI_UPGRADE_MAX);
		else
		{
			_p->player.isLifetimeUpgradeUpdate = TRUE;
			sound_play(_p->g->sounds[SOUND_PURCHASE], -1);
		}
	}
	else
		gui_upgrade_no_cash(_g, _p);
}

/* Upgrade snake level. */
static void
gui_upgrade_redirect(GUI* _g, Play* _p)
{
	if (_p->player.cash >= _p->player.redirectUpgradePrice)
	{
		_p->player.cash -= _p->player.redirectUpgradePrice;

		message_purchase_add(_p->player.sprite.x, _p->player.sprite.y, _p->player.redirectUpgradePrice, _p);

		_p->player.redirectUpgradeLevel++;

		if (_p->player.redirectUpgradeLevel > GUI_UPGRADE_MAX)
			_p->player.redirectUpgradeLevel = CEIL(_p->player.redirectUpgradeLevel, GUI_UPGRADE_MAX);
		else
		{
			_p->player.isRedirectUpgradeUpdate = TRUE;

			_p->player.isRedirectUpdate = TRUE;

			sound_play(_p->g->sounds[SOUND_PURCHASE], -1);
		}
	}
	else
		gui_upgrade_no_cash(_g, _p);
}

/* Sets the gui upgrade text. */
static void 
gui_upgrade_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_UPGRADE_TEXT_MAX);

	if (_g->upgradeText.atlas.texture)
		texture_free(_g->upgradeText.atlas.texture);

	if (_g->snakeUpgradeButton.state == BUTTON_STATE_HOVER)
	{
		snprintf
		(
			s,
			GUI_UPGRADE_TEXT_MAX,
			"%s",
			STRING_PLAY_UPGRADE_SNAKES
		);
	}
	else if (_g->lifetimeUpgradeButton.state == BUTTON_STATE_HOVER)
	{
		snprintf
		(
			s,
			GUI_UPGRADE_TEXT_MAX,
			"%s",
			STRING_PLAY_UPGRADE_LIFETIME
		);
	}
	else if (_g->redirectUpgradeButton.state == BUTTON_STATE_HOVER)
	{
		snprintf
		(
			s,
			GUI_UPGRADE_TEXT_MAX,
			"%s",
			STRING_PLAY_UPGRADE_REDIRECT
		);
	}
	else
	{
		snprintf
		(
			s,
			GUI_UPGRADE_TEXT_MAX,
			"%s",
			STRING_PLAY_UPGRADE
		);
	}

	text_sprite_init
	(
		&_g->upgradeText,
		_p->g->renderer,
		_p->g->fonts[FONT_TINY],
		GUI_UPGRADE_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		GUI_UPGRADE_WRAP
	);

	free(s);

	_g->upgradeText.x = _g->upgradeBackgroundSprite.x;
	_g->upgradeText.y = _g->upgradeBackgroundSprite.y;

	_g->upgradeText.rotation = GUI_UPGRADE_ROTATION;
}

/* Displays upgrade quantity and price of the snake upgrade. */
static void
gui_upgrade_snake_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_UPGRADE_INFO_TEXT_MAX);

	if (_g->snakeUpgradeText.atlas.texture)
		texture_free(_g->snakeUpgradeText.atlas.texture);

	snprintf
	(
		s,
		GUI_UPGRADE_INFO_TEXT_MAX,
		"(x%i) - $%i",
		_p->player.snakeUpgradeLevel,
		_p->player.snakeUpgradePrice
	);

	text_sprite_init
	(
		&_g->snakeUpgradeText,
		_p->g->renderer,
		_p->g->fonts[FONT_SMALL],
		GUI_UPGRADE_INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	free(s);

	_g->snakeUpgradeText.x = _g->snakeUpgradeButton.sprite.x + GUI_UPGRADE_TEXT_OFFSET_X;
	_g->snakeUpgradeText.y = _g->snakeUpgradeButton.sprite.y + GUI_UPGRADE_TEXT_OFFSET_Y;
}

/* Displays upgrade quantity and price of the lifetime upgrade. */
static void
gui_upgrade_lifetime_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_UPGRADE_INFO_TEXT_MAX);

	if (_g->lifetimeUpgradeText.atlas.texture)
		texture_free(_g->lifetimeUpgradeText.atlas.texture);

	snprintf
	(
		s,
		GUI_UPGRADE_INFO_TEXT_MAX,
		"(x%i) - $%i",
		_p->player.lifetimeUpgradeLevel,
		_p->player.lifetimeUpgradePrice
	);

	text_sprite_init
	(
		&_g->lifetimeUpgradeText,
		_p->g->renderer,
		_p->g->fonts[FONT_SMALL],
		GUI_UPGRADE_INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	free(s);

	_g->lifetimeUpgradeText.x = _g->lifetimeUpgradeButton.sprite.x + GUI_UPGRADE_TEXT_OFFSET_X;
	_g->lifetimeUpgradeText.y = _g->lifetimeUpgradeButton.sprite.y + GUI_UPGRADE_TEXT_OFFSET_Y;
}

/* Displays upgrade quantity and price of the redirect upgrade. */
static void
gui_upgrade_redirect_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_UPGRADE_INFO_TEXT_MAX);

	if (_g->redirectUpgradeText.atlas.texture)
		texture_free(_g->redirectUpgradeText.atlas.texture);

	snprintf
	(
		s,
		GUI_UPGRADE_INFO_TEXT_MAX,
		"(x%i) - $%i",
		_p->player.redirectUpgradeLevel,
		_p->player.redirectUpgradePrice
	);

	text_sprite_init
	(
		&_g->redirectUpgradeText,
		_p->g->renderer,
		_p->g->fonts[FONT_SMALL],
		GUI_UPGRADE_INFO_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	free(s);

	_g->redirectUpgradeText.x = _g->redirectUpgradeButton.sprite.x + GUI_UPGRADE_TEXT_OFFSET_X;
	_g->redirectUpgradeText.y = _g->redirectUpgradeButton.sprite.y + GUI_UPGRADE_TEXT_OFFSET_Y;
}


/* Initializes upgrade GUI. */
void 
gui_upgrade_init(GUI* _g, Play* _p)
{
	Sprite snakeUpgradeSprite;
	Sprite redirectUpgradeSprite;
	Sprite lifetimeUpgradeSprite;

	sprite_init
	(
		&snakeUpgradeSprite,
		_p->g->textures[TEXTURE_GUI_UPGRADE_SNAKE],
		ORIGIN_MIDDLE,
		GUI_UPGRADE_WIDTH,
		GUI_UPGRADE_HEIGHT
	);

	sprite_init
	(
		&lifetimeUpgradeSprite,
		_p->g->textures[TEXTURE_GUI_UPGRADE_LIFETIME],
		ORIGIN_MIDDLE,
		GUI_UPGRADE_WIDTH,
		GUI_UPGRADE_HEIGHT
	);

	sprite_init
	(
		&redirectUpgradeSprite,
		_p->g->textures[TEXTURE_GUI_UPGRADE_REDIRECT],
		ORIGIN_MIDDLE,
		GUI_UPGRADE_WIDTH,
		GUI_UPGRADE_HEIGHT
	);

	sprite_init
	(
		&_g->upgradeBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_BIG],
		ORIGIN_MIDDLE,
		GUI_UPGRADE_BACKGROUND_BIG_WIDTH,
		GUI_UPGRADE_BACKGROUND_BIG_HEIGHT
	);

	sprite_init
	(
		&_g->snakeUpgradeBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_SMALL],
		ORIGIN_MIDDLE,
		GUI_UPGRADE_BACKGROUND_SMALL_WIDTH,
		GUI_UPGRADE_BACKGROUND_SMALL_HEIGHT
	);

	sprite_init
	(
		&_g->lifetimeUpgradeBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_SMALL],
		ORIGIN_MIDDLE,
		GUI_UPGRADE_BACKGROUND_SMALL_WIDTH,
		GUI_UPGRADE_BACKGROUND_SMALL_HEIGHT
	);

	sprite_init
	(
		&_g->redirectUpgradeBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_SMALL],
		ORIGIN_MIDDLE,
		GUI_UPGRADE_BACKGROUND_SMALL_WIDTH,
		GUI_UPGRADE_BACKGROUND_SMALL_HEIGHT
	);

	sprite_init
	(
		&_g->readyBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_SMALL],
		ORIGIN_MIDDLE,
		GUI_UPGRADE_BACKGROUND_SMALL_WIDTH,
		GUI_UPGRADE_BACKGROUND_SMALL_HEIGHT
	);

	_g->upgradeBackgroundSprite.x = GUI_UPGRADE_BACKGROUND_SPRITE_OFFSET_X;
	_g->upgradeBackgroundSprite.y = GUI_UPGRADE_BACKGROUND_SPRITE_OFFSET_Y;

	_g->snakeUpgradeBackgroundSprite.x = GUI_UPGRADE_SNAKE_SPRITE_OFFSET_X;
	_g->lifetimeUpgradeBackgroundSprite.x = GUI_UPGRADE_LIFETIME_SPRITE_OFFSET_X;
	_g->redirectUpgradeBackgroundSprite.x = GUI_UPGRADE_REDIRECT_SPRITE_OFFSET_X;
	_g->snakeUpgradeBackgroundSprite.y = GUI_UPGRADE_SNAKE_SPRITE_OFFSET_Y;
	_g->lifetimeUpgradeBackgroundSprite.y = GUI_UPGRADE_LIFETIME_SPRITE_OFFSET_Y;
	_g->redirectUpgradeBackgroundSprite.y = GUI_UPGRADE_REDIRECT_SPRITE_OFFSET_Y;

	button_init(&_g->snakeUpgradeButton, &snakeUpgradeSprite);
	button_init(&_g->lifetimeUpgradeButton, &lifetimeUpgradeSprite);
	button_init(&_g->redirectUpgradeButton, &redirectUpgradeSprite);

	_g->snakeUpgradeButton.sprite.x = _g->snakeUpgradeBackgroundSprite.x;
	_g->snakeUpgradeButton.sprite.y = _g->snakeUpgradeBackgroundSprite.y;
	_g->lifetimeUpgradeButton.sprite.x = _g->lifetimeUpgradeBackgroundSprite.x;
	_g->lifetimeUpgradeButton.sprite.y = _g->lifetimeUpgradeBackgroundSprite.y;
	_g->redirectUpgradeButton.sprite.x = _g->redirectUpgradeBackgroundSprite.x;
	_g->redirectUpgradeButton.sprite.y = _g->redirectUpgradeBackgroundSprite.y;

	gui_upgrade_text_set(_g, _p);
	gui_upgrade_snake_text_set(_g, _p);
	gui_upgrade_lifetime_text_set(_g, _p);
	gui_upgrade_redirect_text_set(_g, _p);
}

/* Frees upgrade GUI. */
void 
gui_upgrade_free(GUI* _g, Play* _p)
{
	if (_g->upgradeText.atlas.texture)
		texture_free(_g->upgradeText.atlas.texture);

	if (_g->snakeUpgradeText.atlas.texture)
		texture_free(_g->snakeUpgradeText.atlas.texture);

	if (_g->lifetimeUpgradeText.atlas.texture)
		texture_free(_g->lifetimeUpgradeText.atlas.texture);

	if (_g->redirectUpgradeText.atlas.texture)
		texture_free(_g->redirectUpgradeText.atlas.texture);
}

/* Updates upgrade GUI. */
void 
gui_upgrade_update(GUI* _g, Play* _p)
{
	if (_p->isUpgradeUpdate)
		_p->isUpgradeUpdate = FALSE;

	if (_p->player.isSnakeUpgradeUpdate)
	{
		gui_upgrade_snake_text_set(_g, _p);
		_p->player.isSnakeUpgradeUpdate = FALSE;
	}

	if (_p->player.isLifetimeUpgradeUpdate)
	{
		gui_upgrade_lifetime_text_set(_g, _p);
		_p->player.isLifetimeUpgradeUpdate = FALSE;
	}

	if (_p->player.isRedirectUpgradeUpdate)
	{
		gui_upgrade_redirect_text_set(_g, _p);
		_p->player.isRedirectUpgradeUpdate = FALSE;
	}

	if (_p->isUpgradesActive)
	{
		button_update(&_g->snakeUpgradeButton, &_p->g->mouse);
		button_update(&_g->lifetimeUpgradeButton, &_p->g->mouse);
		button_update(&_g->redirectUpgradeButton, &_p->g->mouse);

		if (_g->snakeUpgradeButton.state == BUTTON_STATE_PRESS)
			gui_upgrade_snake(_g, _p);

		if (_g->lifetimeUpgradeButton.state == BUTTON_STATE_PRESS)
			gui_upgrade_lifetime(_g, _p);

		if (_g->redirectUpgradeButton.state == BUTTON_STATE_PRESS)
			gui_upgrade_redirect(_g, _p);
		
		if
		(
			_g->snakeUpgradeButton.state == BUTTON_STATE_HOVER ||
			_g->lifetimeUpgradeButton.state == BUTTON_STATE_HOVER ||
			_g->redirectUpgradeButton.state == BUTTON_STATE_HOVER
		)
			_p->isUpgradeUpdate = TRUE;

		if (_p->isUpgradeUpdate)
			gui_upgrade_text_set(_g, _p);
	}
}

/* Draws upgrade GUI. */
void 
gui_upgrade_draw(GUI* _g, Play* _p)
{
	if (_p->isUpgradesActive)
	{
		sprite_draw(&_g->upgradeBackgroundSprite, _p->g->renderer);
		sprite_draw(&_g->upgradeText, _p->g->renderer);

		sprite_draw(&_g->snakeUpgradeBackgroundSprite, _p->g->renderer);
		sprite_draw(&_g->lifetimeUpgradeBackgroundSprite, _p->g->renderer);
		sprite_draw(&_g->redirectUpgradeBackgroundSprite, _p->g->renderer);

		button_draw(&_g->snakeUpgradeButton, _p->g->renderer);
		button_draw(&_g->lifetimeUpgradeButton, _p->g->renderer);
		button_draw(&_g->redirectUpgradeButton, _p->g->renderer);

		sprite_draw(&_g->snakeUpgradeText, _p->g->renderer);
		sprite_draw(&_g->lifetimeUpgradeText, _p->g->renderer);
		sprite_draw(&_g->redirectUpgradeText, _p->g->renderer);
	}
}
