/*
 * DESCRIPTION:
 * Displays a wave icon and the current waves available.
*/

#include "gui_wave.h"

/* Sets the gui wave text. */
static void 
gui_wave_text_set(GUI* _g, Play* _p)
{
	char* s;

	s = malloc(GUI_WAVE_TEXT_MAX);

	if (_g->waveText.atlas.texture)
		texture_free(_g->waveText.atlas.texture);

	if (_p->isBetweenWaves)
	{
		snprintf
		(
			s,
			GUI_WAVE_TEXT_MAX,
			"%s-",
			STRING_PLAY_GUI_WAVE
		);
	}
	else
	{
		if (_p->game > 1)
		{
			snprintf
			(
				s,
				GUI_WAVE_TEXT_MAX,
				"%s%i (NG+%i)",
				STRING_PLAY_GUI_WAVE,
				_p->wave,
				_p->game - 1
			);
			}
		else
		{
			snprintf
			(
				s,
				GUI_WAVE_TEXT_MAX,
				"%s%i",
				STRING_PLAY_GUI_WAVE,
				_p->wave
			);
		}
	}

	text_sprite_init
	(
		&_g->waveText,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		GUI_WAVE_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	_g->waveText.x = _g->waveBackgroundSprite.x;
	_g->waveText.y = _g->waveBackgroundSprite.y;

	_g->waveText.rotation = GUI_WAVE_TEXT_ROTATION;
	_g->waveText.color.a = GUI_WAVE_ALPHA;

	free(s);
}

/* Initializes wave GUI. */
void 
gui_wave_init(GUI* _g, Play* _p)
{
	sprite_init
	(
		&_g->waveBackgroundSprite,
		_p->g->textures[TEXTURE_GUI_SIGN_BIG],
		ORIGIN_MIDDLE,
		GUI_WAVE_BACKGROUND_WIDTH,
		GUI_WAVE_BACKGROUND_HEIGHT
	);

	_g->waveBackgroundSprite.flip = FLIP_HORIZONTAL;

	_g->waveBackgroundSprite.x = GUI_WAVE_SPRITE_OFFSET_X;
	_g->waveBackgroundSprite.y = GUI_WAVE_SPRITE_OFFSET_Y;

	_g->waveBackgroundSprite.color.a = GUI_WAVE_ALPHA;

	gui_wave_text_set(_g, _p);
}

/* Frees wave GUI. */
void 
gui_wave_free(GUI* _g, Play* _p)
{
	if (_g->waveText.atlas.texture)
		texture_free(_g->waveText.atlas.texture);
}

/* Updates wave GUI. */
void 
gui_wave_update(GUI* _g, Play* _p)
{
	if (_p->isWaveUpdate)
		gui_wave_text_set(_g, _p);
}

/* Draws wave GUI. */
void 
gui_wave_draw(GUI* _g, Play* _p)
{
	sprite_draw(&_g->waveBackgroundSprite, _p->g->renderer);
	sprite_draw(&_g->waveText, _p->g->renderer);
}
