/*
 * DESCRIPTION:
 * Messages, or in-game blurbs that display happenings.
*/

#include "message.h"

static Message* message_add(Message* _m, f64 _x, f64 _y, Play* _p);

/* Adds a message to the list. */
static Message*
message_add(Message* _m, f64 _x, f64 _y, Play* _p)
{
	_m->sprite.x = _x;
	_m->sprite.y = _y;
	_m->dR = 0;
	_m->dX = 0;
	_m->dY = MESSAGE_VELOCITY;

	_m->lifetime = MESSAGE_LIFETIME;

	return (Message*)list_append_new(&_p->messages, (void*)_m, sizeof(Message));
}

/* +$X */
Message*
message_cash_add(f64 _x, f64 _y, s32 _c, Play* _p)
{
	Message m;
	char* s;

	s = malloc(MESSAGE_TEXT_MAX);

	snprintf
	(
		s,
		MESSAGE_TEXT_MAX,
		"+$%i",
		_c
	);

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_CASH_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	free(s);

	return message_add(&m, _x, _y, _p);
}

/* -$X */
Message*
message_purchase_add(f64 _x, f64 _y, s32 _c, Play* _p)
{
	Message m;
	char* s;

	s = malloc(MESSAGE_TEXT_MAX);

	snprintf
	(
		s,
		MESSAGE_TEXT_MAX,
		"-$%i",
		_c
	);

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_PURCHASE_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	free(s);

	return message_add(&m, _x, _y, _p);
}

/* "You're broke!!" */
Message*
message_no_cash_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_PURCHASE_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_PURCHASE_NO_CASH,
		0
	);

	return message_add(&m, _x, _y, _p);
}

/* "MAXED!!" */
Message*
message_maxed_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_PURCHASE_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_PURCHASE_MAXED,
		0
	);

	return message_add(&m, _x, _y, _p);
}

/* -X */
Message*
message_damage_add(f64 _x, f64 _y, s32 _d, Play* _p)
{
	Message m;
	char* s;

	s = malloc(MESSAGE_TEXT_MAX);

	snprintf
	(
		s,
		MESSAGE_TEXT_MAX,
		"-%i",
		_d
	);

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_DAMAGE_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	free(s);

	return message_add(&m, _x, _y, _p);
}

/* "Wave cleared!" */
Message*
message_wave_clear_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_WAVE_CLEAR_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_WAVE_CLEAR,
		0
	);

	return message_add(&m, _x, _y, _p);
}

/* "Wave failed..." */
Message*
message_wave_fail_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_WAVE_FAIL_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_WAVE_FAIL,
		0
	);

	return message_add(&m, _x, _y, _p);
}

/* "Game cleared!!" */
Message*
message_game_clear_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_GAME_CLEAR_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_GAME_CLEAR,
		0
	);

	return message_add(&m, _x, _y, _p);
}


/* "BERSERK!!" */
Message*
message_power_up_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_POWER_UP_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_POWER_UP,
		0
	);

	return message_add(&m, _x, _y, _p);
}


/* "Berserk over..." */
Message*
message_power_down_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_POWER_DOWN_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_POWER_DOWN,
		0
	);

	return message_add(&m, _x, _y, _p);
}

/* "-1 Fatness!!" */
Message*
message_miracle_cure_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_MIRACLE_CURE_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_MIRACLE_CURE,
		0
	);

	return message_add(&m, _x, _y, _p);
}

/* "KABOOM!!" */
Message*
message_dynamite_add(f64 _x, f64 _y, Play* _p)
{
	Message m;

	text_sprite_init
	(
		&m.sprite,
		_p->g->renderer,
		_p->g->fonts[FONT_MEDIUM],
		MESSAGE_DYNAMITE_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_MESSAGE_DYNAMITE,
		0
	);

	return message_add(&m, _x, _y, _p);
}

/* Updates a message. */
void
message_update(Message* _m, Play* _p)
{
	f64 progress;

	progress = ((f64)_m->lifetime / MESSAGE_LIFETIME);

	_m->lifetime--;

	_m->sprite.x += _m->dX;
	_m->sprite.y += _m->dY;
	_m->sprite.rotation += _m->dR;

	_m->sprite.color.a = SDL_ALPHA_OPAQUE * progress;

	if (_m->lifetime <= 0)
		list_node_from_data_remove(&_p->messages, _m, sizeof(Message));
}

/* Draws a message. */
void
message_draw(Message* _m, Play* _p)
{
	sprite_draw(&_m->sprite, _p->g->renderer);
}
