/*
 * DESCRIPTION:
 * Main gameplay functions. 
*/

#include "play.h"

/* Initializes gameplay resources. */
void
play_init(Play* _p, Game* _g)
{
	memset(_p, '\0', sizeof(Play));

	_p->g = _g;
	
	music_play(_p->g->music[MUSIC_PLAY], TRUE);

	player_init(&_p->player, _p);
	enemies_init(&_p->enemies, _p);
	collectibles_init(&_p->collectibles, _p);

	wave_init(_p);
	spawn_init(_p);
	pause_init(_p);

	gui_init(&_p->gui, _p);
	messages_init(_p);

	tutorial_init(_p);

	text_sprite_init
	(
		&_p->pauseText,
		_p->g->renderer,
		_p->g->fonts[FONT_MEGA],
		PLAY_PAUSE_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_PLAY_PAUSE,
		0
	);

	_p->pauseText.x = PLAY_PAUSE_X_OFFSET;
	_p->pauseText.y = PLAY_PAUSE_Y_OFFSET;

	sprite_init
	(
		&_p->bg,
		_p->g->textures[TEXTURE_PLAY_BG],
		ORIGIN_TOP_LEFT,
		PLAY_BG_WIDTH,
		PLAY_BG_HEIGHT
	);

	_p->bg.x = PLAY_BG_OFFSET_X;
	_p->bg.y = PLAY_BG_OFFSET_Y;
}

/* Frees gameplay resources. */
void
play_free(Play* _p)
{
	enemies_free(&_p->enemies, _p);
	collectibles_free(&_p->collectibles, _p);
	player_free(&_p->player, _p);
	gui_free(&_p->gui, _p);
	messages_free(_p);
	tutorial_free(_p);
}

/* Updates gameplay. */
void
play_update(Play* _p)
{
	gui_update(&_p->gui, _p);
	pause_update(_p);

	if (!_p->isPaused)
	{
		player_update(&_p->player, _p);
		enemies_update(&_p->enemies, _p);
		collectibles_update(&_p->collectibles, _p);
		messages_update(_p);

		wave_update(_p);
		spawn_update(_p);

		tutorial_update(_p);
	}
}

/* Draws gameplay. */
void
play_draw(Play* _p)
{
	sprite_draw(&_p->bg, _p->g->renderer);
	enemies_draw(&_p->enemies, _p);
	collectibles_draw(&_p->collectibles, _p);
	player_draw(&_p->player, _p);
	messages_draw(_p);
	tutorial_draw(_p);
	gui_draw(&_p->gui, _p);

	if (_p->isPaused)
	{
		SDL_Rect r;

		r.x = 0;
		r.y = 0;
		r.w = PLAY_PAUSE_RECTANGLE_WIDTH;
		r.h = PLAY_PAUSE_RECTANGLE_HEIGHT;

		rectangle_draw(&r, _p->g->renderer, PLAY_PAUSE_BACKGROUND_COLOR);

		sprite_draw(&_p->pauseText, _p->g->renderer);
	}
}
