/* 
 * DESCRIPTION:
 * Functions concering player fatness.
*/

#include "player_fatness.h"

static void player_fatness_swallow(Player* _p, Play* _l);
static void player_fatness_gain(Player* _p, Play* _l);

/* When you hit an enemy, you 'swallow" them, and this happens. */
static void
player_fatness_swallow(Player* _p, Play* _l)
{
	enemies_defeat(&_l->enemies, _l);
	list_clear(&_l->spawnEnemies);

	_l->isWaveFailed = TRUE;

	_p->isAbleToMove = FALSE;
	_p->isAbleToFire = FALSE;

	_p->dX = 0;
	_p->dY = 0;

	_p->isBeingFattened = TRUE;
	_p->fatnessTimer = PLAYER_FATNESS_TIMER_MAX;

	_p->isFatnessUpdate = TRUE;

	_l->enemyMax = 0;
	_l->isBountyUpdate = TRUE;

	_p->isStationary = TRUE;

	sound_play(_l->g->sounds[SOUND_SWALLOW], -1);

	if (_p->fatness + 1 > PLAYER_FATNESS_THRESHOLD)
		player_defeat(_p, _l);
}

/* You gain fatness. */
static void
player_fatness_gain(Player* _p, Play* _l)
{
	_p->isAbleToMove = TRUE;
	_p->isAbleToFire = TRUE;

	_p->fatness++;
	_p->isBeingFattened = FALSE;
	_p->isFatnessUpdate = TRUE;

	sound_play(_l->g->sounds[SOUND_GROWTH], -1);
}

/* Initializes player fatness. */
void
player_fatness_init(Player* _p, Play* _l)
{
	_p->fatness = PLAYER_FATNESS_DEFAULT;
}

/* Updates the player fatness. */
void
player_fatness_update(Player* _p, Play* _l)
{
	Enemy** enemies;
	s32 enemyCount;

	if (_p->isFatnessUpdate)
		_p->isFatnessUpdate = FALSE;

	enemies = (Enemy**)list_data_get(&_l->enemies);
	enemyCount = _l->enemies.count;

	for (s32 i = 0; i < enemyCount; i++)
	{
		Enemy* e;

		e = enemies[i];

		if 
		(
			circle_collide(&_p->shadow, &e->shadow) &&
			e->isActive
		)
		{
			player_fatness_swallow(_p, _l);
			break;
		}
	}

	if (_p->isBeingFattened)
	{
		_p->fatnessTimer--;

		if (_p->fatnessTimer <= 0)
			player_fatness_gain(_p, _l);
	}

	free(enemies);

	_p->fatness = CLAMP(_p->fatness, 0, PLAYER_FATNESS_MAX - 1);
}
