/*
 * DESCRIPTION:
 * Player firing code.
*/

#include "player_fire.h"

static void player_fire(Player* _p, Play* _l);
static void player_redirect(Player* _p, Play* _l);

/* Fires a snake. */
static void
player_fire(Player* _p, Play* _l)
{
	player_snake_add(_p, _l);

	sound_play(_l->g->sounds[SOUND_SNAKE_FIRE], -1);

	_p->isSnakeUpdate = TRUE;
}

/* Redirects all snakes. The angle of the snakes must be significant enough to do so. */
static void
player_redirect(Player* _p, Play* _l)
{
	Snake** snakes;
	Snake** snakesAvailable;
	s32 snakesAvailableCount;

	snakes = (Snake**)list_data_get(&_p->snakes);

	snakesAvailableCount = 0;
	snakesAvailable = NULL;

	for (s32 i = 0; i < _p->snakes.count; i++)
	{
		f64 playerAngle;
		f64 snakeAngle;
		f64 angleDifference;
		Snake* s;

		s = snakes[i];

		playerAngle = math_atan
		(
			_l->player.shadow.x,
			_l->player.shadow.y,
			_l->g->mouse.x,
			_l->g->mouse.y
		);

		snakeAngle = math_atan
		(
		 	_l->player.shadow.x,
			_l->player.shadow.y,
			s->sprite.x,
			s->sprite.y
		);

		playerAngle *= RAD_TO_DEG;
		snakeAngle *= RAD_TO_DEG;

		angleDifference = fabs(playerAngle - snakeAngle);

		if 
		(
			angleDifference > PLAYER_SNAKE_REDIRECT_ANGLE_DIFFERENCE_MIN &&
			!s->isReturning
		)
		{
			snakesAvailableCount++;
	
			snakesAvailable = realloc(snakesAvailable, sizeof(Snake*) * snakesAvailableCount);

			snakesAvailable[snakesAvailableCount - 1] = s;
		}
	}

	if (snakesAvailableCount > 0)
		_p->redirects--;

	for (s32 i = 0; i < snakesAvailableCount; i++)
	{
		Snake* s;

		s = snakesAvailable[i];

		s->redirectCount++;

		s->redirectCount = CEIL(s->redirectCount, PLAYER_REDIRECT_MAX);

		s->redirectTimer = PLAYER_REDIRECT_TIMER_MAX;
		s->sprite.frame = SNAKE_REDIRECT;
		
		s->damage = PLAYER_SNAKE_DAMAGE_DEFAULT + pow(s->redirectCount, 2);

		player_snake_reset(s, _p, _l);
	}

	free(snakes);

	_p->isRedirectUpdate = TRUE;

	if (snakesAvailableCount > 0)
		sound_play(_l->g->sounds[SOUND_SNAKE_REDIRECT], 7);
}

/* Initializes player firing. */
void
player_fire_init(Player* _p, Play* _l)
{
	_p->isAbleToFire = TRUE;
	_p->snakeReserve = _p->snakeMax;
	_p->redirects = _p->redirectMax;

	_p->redirectRechargeTimer = PLAYER_REDIRECT_RECHARGE_TIMER_MAX;
}

/* Updates the player's firing. */
void
player_fire_update(Player* _p, Play* _l)
{
	if (_p->isSnakeUpdate)
		_p->isSnakeUpdate = FALSE;

	if (_p->isRedirectUpdate)
		_p->isRedirectUpdate = FALSE;

	if (_p->redirectTimer > 0)
		_p->redirectTimer--;


	if (_p->snakeReserve > 0)
	{
		if (_p->isAbleToFire)
		{
			if (mouse_press(&_l->g->mouse, MOUSE_LEFT))
			{
				player_fire(_p, _l);
				_p->snakeReserve--;
			}
		}
	}
	if  
	(
		_p->snakes.count > 0 &&
		_p->redirects > 0 &&
		_p->redirectTimer <= 0
	)
		if 
		(
			mouse_press(&_l->g->mouse, MOUSE_RIGHT) ||
			keyboard_press(&_l->g->keyboard, KEYBOARD_R)
		)
		{
			player_redirect(_p, _l);
			_p->redirectTimer = PLAYER_REDIRECT_TIMER_MAX;
		}

	if (_p->snakes.count == 0)
	{
		_p->redirects = _p->redirectMax;
		_p->isRedirectUpdate = TRUE;
	}
}
