/*
 * DESCRIPTION:
 * Player movement code.
*/

#include "player_movement.h"

static void player_velocity_reset(Player* _p);
static bool_u8 player_movement_left_held(Play* _p);
static bool_u8 player_movement_right_held(Play* _p);
static bool_u8 player_movement_up_held(Play* _p);
static bool_u8 player_movement_down_held(Play* _p);

static bool_u8
player_movement_left_held(Play* _p)
{
	if 
	(
		keyboard_held(&_p->g->keyboard, KEYBOARD_LEFT) ||
		keyboard_held(&_p->g->keyboard, KEYBOARD_A)
	)
		return TRUE;
	return FALSE;
}

static bool_u8
player_movement_right_held(Play* _p)
{
	if 
	(
		keyboard_held(&_p->g->keyboard, KEYBOARD_RIGHT) ||
		keyboard_held(&_p->g->keyboard, KEYBOARD_D)
	)
		return TRUE;
	return FALSE;
}

static bool_u8
player_movement_up_held(Play* _p)
{
	if 
	(
		keyboard_held(&_p->g->keyboard, KEYBOARD_UP) ||
		keyboard_held(&_p->g->keyboard, KEYBOARD_W)
	)
		return TRUE;
	return FALSE;
}

static bool_u8
player_movement_down_held(Play* _p)
{
	if 
	(
		keyboard_held(&_p->g->keyboard, KEYBOARD_DOWN) ||
		keyboard_held(&_p->g->keyboard, KEYBOARD_S)
	)
		return TRUE;
	return FALSE;
}

static void
player_velocity_reset(Player* _p)
{
	_p->dX = 0;
	_p->dY = 0;
	_p->isStationary = TRUE;
}

void
player_movement_init(Player* _p, Play* _l)
{
	_p->isAbleToMove = TRUE;
	_p->isKeyboardMove = TRUE;;
}

/* Updates player movement. */
void
player_movement_update(Player* _p, Play* _l)
{
	if (_p->isStationary)
		_p->isStationary = FALSE;

	if (_p->isAbleToMove && !_l->isGameCleared)
	{
		f64 angle;
		f64 velocity;

		velocity = PLAYER_VELOCITIES[_p->fatness];

		if (_p->isKeyboardMove)
		{
			bool_u8 isMove;

			_p->isStationary = TRUE;
			isMove = FALSE;

			player_velocity_reset(_p);

			if (player_movement_left_held(_l))
			{
				angle = PLAYER_MOVEMENT_LEFT_ANGLE;
				_p->direction = DIRECTION_LEFT;

				if (player_movement_up_held(_l))
					angle = PLAYER_MOVEMENT_LEFT_UP_ANGLE;
				else if (player_movement_down_held(_l))
					angle = PLAYER_MOVEMENT_LEFT_DOWN_ANGLE;

				isMove = TRUE;
			}
			else if (player_movement_right_held(_l))
			{
				angle = PLAYER_MOVEMENT_RIGHT_ANGLE;
				_p->direction = DIRECTION_RIGHT;

				if (player_movement_up_held(_l))
					angle = PLAYER_MOVEMENT_RIGHT_UP_ANGLE;
				else if (player_movement_down_held(_l))
					angle = PLAYER_MOVEMENT_RIGHT_DOWN_ANGLE;

				isMove = TRUE;
			}
			else if (player_movement_up_held(_l))
			{
				angle = PLAYER_MOVEMENT_UP_ANGLE;
				//_p->direction = DIRECTION_UP;
				isMove = TRUE;
			}
			else if (player_movement_down_held(_l))
			{
				angle = PLAYER_MOVEMENT_DOWN_ANGLE;
				//_p->direction = DIRECTION_DOWN;
				isMove = TRUE;
			}

			_p->dX = -(cos(angle) * velocity);
			_p->dY = -(sin(angle) * velocity);

			if (isMove)
				_p->isStationary = FALSE;
			else
				player_velocity_reset(_p);

		}
		else
		{
			if (!mouse_held(&_l->g->mouse, MOUSE_RIGHT))
				player_velocity_reset(_p);
			else
			{
				f64 distance;

				distance = math_distance
				(
					_p->sprite.x,
					_p->sprite.y,
					_l->g->mouse.x,
					_l->g->mouse.y
				);

				if (distance > PLAYER_MOVE_MINIMUM)
				{
					angle = math_atan
					(
						_p->sprite.x,
						_p->sprite.y,
						_l->g->mouse.x,
						_l->g->mouse.y
					);

					_p->dX = -(cos(angle) * velocity);
					_p->dY = -(sin(angle) * velocity);

					_p->isStationary = FALSE;
				}
				else
					player_velocity_reset(_p);
			}
		}
	}

        _p->sprite.x += _p->dX;
        _p->sprite.y += _p->dY;

	_p->sprite.x = _p->sprite.x < PLAYER_MOVEMENT_X_MIN ? PLAYER_MOVEMENT_X_MIN : _p->sprite.x;
	_p->sprite.x = _p->sprite.x > PLAYER_MOVEMENT_X_MAX ? PLAYER_MOVEMENT_X_MAX : _p->sprite.x;
	_p->sprite.y = _p->sprite.y < PLAYER_MOVEMENT_Y_MIN ? PLAYER_MOVEMENT_Y_MIN : _p->sprite.y;
	_p->sprite.y = _p->sprite.y > PLAYER_MOVEMENT_Y_MAX ? PLAYER_MOVEMENT_Y_MAX : _p->sprite.y;
}
