/*
 * DESCRIPTION:
 * Determines prices for player upgrades.
*/

#include "player_price.h"

static s32 player_price_snake_get(Player* _p, Play* _l);
static s32 player_price_redirect_get(Player* _p, Play* _l);
static s32 player_price_lifetime_get(Player* _p, Play* _l);

/* Gets the snake upgrade price. */
static s32 
player_price_snake_get(Player* _p, Play* _l)
{
	s32 price;

	price = (s32)
	(
		pow
		(
			PLAYER_PRICE_BASE, 
			PLAYER_PRICE_EXPONENT_BASE + ((_p->snakeUpgradeLevel - 1) * PLAYER_PRICE_EXPONENT_INCREMENT)
		)
	);

	return price;
}

/* Gets the lifetime upgrade price. */
static s32
player_price_lifetime_get(Player* _p, Play* _l)
{
	s32 price;

	price = (s32)
	(
		pow
		(
			PLAYER_PRICE_BASE, 
			PLAYER_PRICE_EXPONENT_BASE + ((_p->lifetimeUpgradeLevel - 1) * PLAYER_PRICE_EXPONENT_INCREMENT)
		)
	);

	return price;
}

/* Gets the redirect upgrade price. */
static s32 
player_price_redirect_get(Player* _p, Play* _l)
{
	s32 price;

	price = (s32)
	(
		pow
		(
			PLAYER_PRICE_BASE, 
			PLAYER_PRICE_EXPONENT_BASE + ((_p->redirectUpgradeLevel - 1) * PLAYER_PRICE_EXPONENT_INCREMENT)
		)
	);

	return price;
}

/* Initializes player prices. */
void 
player_price_init(Player* _p, Play* _l)
{
	_p->snakeUpgradePrice = player_price_snake_get(_p, _l);
	_p->lifetimeUpgradePrice = player_price_lifetime_get(_p, _l);
	_p->redirectUpgradePrice = player_price_redirect_get(_p, _l);
}

/* Updates player prices. */
void
player_price_update(Player* _p, Play* _l)
{
	if (_p->isSnakeUpgradeUpdate)
		_p->snakeUpgradePrice = player_price_snake_get(_p, _l);

	if (_p->isLifetimeUpgradeUpdate)
		_p->lifetimeUpgradePrice = player_price_lifetime_get(_p, _l);

	if (_p->isRedirectUpgradeUpdate)
		_p->redirectUpgradePrice = player_price_redirect_get(_p, _l);
	

}
