/* 
 * DESCRIPTION:
 * Functions concering player tail.
*/

#include "player_tail.h"

static void player_tail_set(Player* _p, Play* _l);

/* Sets tail's position/dimensions behind player. */
void
player_tail_set(Player* _p, Play* _l)
{
	switch (_p->direction)
	{
		case DIRECTION_LEFT:
			_p->tailSprite.x = _p->sprite.x + PLAYER_TAIL_LEFT_OFFSET_X;
			_p->tailSprite.y = _p->sprite.y + PLAYER_TAIL_LEFT_OFFSET_Y;
			break;
		case DIRECTION_RIGHT:
			_p->tailSprite.x = _p->sprite.x + PLAYER_TAIL_RIGHT_OFFSET_X;
			_p->tailSprite.y = _p->sprite.y + PLAYER_TAIL_RIGHT_OFFSET_Y;
			break;
		/*
		case DIRECTION_UP:
			_p->tailSprite.x = _p->sprite.x + PLAYER_TAIL_UP_OFFSET_X;
			_p->tailSprite.y = _p->sprite.y + PLAYER_TAIL_UP_OFFSET_Y;
			break;
		case DIRECTION_DOWN:
			_p->tailSprite.x = _p->sprite.x + PLAYER_TAIL_DOWN_OFFSET_X;
			_p->tailSprite.y = _p->sprite.y + PLAYER_TAIL_DOWN_OFFSET_Y;
			break;
		*/
		default:
			break;
	}
}

/* Initializes player tail. */
void
player_tail_init(Player* _p, Play* _l)
{
	Atlas atlas;

	atlas_init
	(
		&atlas,
		_l->g->textures[TEXTURE_PLAYER_TAIL_STAGE_ONE],
		PLAYER_TAIL_ROWS,
		PLAYER_TAIL_COLUMNS,
		PLAYER_TAIL_WIDTH,
		PLAYER_TAIL_HEIGHT
	);

        sprite_atlas_init
        (
                &_p->tailSprite,
		&atlas,
                ORIGIN_MIDDLE,
                PLAYER_TAIL_WIDTH,
                PLAYER_TAIL_HEIGHT
        );

	player_tail_set(_p, _l);
}

/* Frees player tail. */
void
player_tail_free(Player* _p, Play* _l)
{


}

/* Updates the player tail. */
void
player_tail_update(Player* _p, Play* _l)
{
	s32 frame;

	if (_p->isAbleToMove)
	{
		frame = _p->direction * PLAYER_TAIL_ANIMATE_COUNT;

		if (_p->isTailAnimate)
			frame += 1;

		_p->tailSprite.frame = frame;
		
		if (!_p->isStationary)
		{
			_p->tailTimer--;

			if (_p->tailTimer <= 0)
			{
				_p->tailTimer = PLAYER_TAIL_TIMER_MAX * (_p->fatness + 1);
				_p->isTailAnimate = !_p->isTailAnimate;
			}
		}
	}

	if (_p->isFatnessUpdate)
	{
		switch (_p->fatness)
		{
			case 0:
				_p->tailSprite.atlas.texture = _l->g->textures[TEXTURE_PLAYER_TAIL_STAGE_ONE];
				break;
			case 1:
				_p->tailSprite.atlas.texture = _l->g->textures[TEXTURE_PLAYER_TAIL_STAGE_TWO];
				break;
			case 2:
				_p->tailSprite.atlas.texture = _l->g->textures[TEXTURE_PLAYER_TAIL_STAGE_THREE];
				break;
			default:
				break;
		}
	}

	player_tail_set(_p, _l);
}

/* Draws the player tail. */
void
player_tail_draw(Player* _p, Play* _l)
{
	//if (!_p->isBeingFattened && !_p->isBeingDefeated)
		sprite_draw(&_p->tailSprite, _l->g->renderer);
}
