/*
 * DESCRIPTION:
 * Dictates player upgrades (redirect, lifetime, snakes)
*/

#include "player_upgrades.h"

static s32 player_upgrades_snake_get(Player* _p, Play* _l);
static s32 player_upgrades_lifetime_get(Player* _p, Play* _l);
static s32 player_upgrades_redirect_get(Player* _p, Play* _l);

/* Gets the snakes the player can have. */
static s32
player_upgrades_snake_get(Player* _p, Play* _l)
{
	s32 snakes;
	s32 upgradeLevel;

	upgradeLevel = _p->snakeUpgradeLevel;

	snakes = upgradeLevel;

	return snakes;
}

/* Gets the lifetime of the snakes the player has. */
static s32
player_upgrades_lifetime_get(Player* _p, Play* _l)
{
	s32 lifetime;
	s32 upgradeLevel;

	upgradeLevel = _p->lifetimeUpgradeLevel;

	lifetime = ((upgradeLevel - 1) * PLAYER_UPGRADES_LIFETIME_MULTIPLIER) + PLAYER_UPGRADES_LIFETIME_BASE;

	return lifetime;
}

/* Gets the redirects the player can have. */
static s32
player_upgrades_redirect_get(Player* _p, Play* _l)
{
	s32 redirects;
	s32 upgradeLevel;

	upgradeLevel = _p->redirectUpgradeLevel;

	redirects = upgradeLevel;

	return redirects;
}

/* Initializes player upgrades. */
void
player_upgrades_init(Player* _p, Play* _l)
{
	_p->snakeUpgradeLevel = PLAYER_UPGRADE_LEVEL_BASE;
	_p->redirectUpgradeLevel = PLAYER_UPGRADE_LEVEL_BASE;
	_p->lifetimeUpgradeLevel = PLAYER_UPGRADE_LEVEL_BASE;

	_p->snakeMax = player_upgrades_snake_get(_p, _l);
	_p->redirectMax = player_upgrades_redirect_get(_p, _l);
	_p->lifetime = player_upgrades_lifetime_get(_p, _l);
}

/* Updates player upgrades. */
void
player_upgrades_update(Player* _p, Play* _l)
{
	if (_p->isSnakeUpgradeUpdate)
	{
		_p->snakeMax = player_upgrades_snake_get(_p, _l);
		_p->snakeReserve = _p->snakeMax;
	}

	if (_p->isRedirectUpgradeUpdate)
	{
		_p->redirectMax = player_upgrades_redirect_get(_p, _l);
		_p->redirects = _p->redirectMax;
	}

	if (_p->isLifetimeUpgradeUpdate)
		_p->lifetime = player_upgrades_lifetime_get(_p, _l);
}
