/*
 * DESCRIPTION:
 * Spawns enemies.
*/

#include "spawn.h"

static s32 spawn_timer_max_get(Play* _p);

/* Gets a new spawn timer max, influenced by the wave. */
static s32
spawn_timer_max_get(Play* _p)
{
	s32 spawnTimerBase;
	s32 spawnTimerMax;

	spawnTimerBase = SPAWN_TIMER_MAX_DEFAULT - ((_p->wave * _p->game) * SPAWN_TIMER_WAVE_MULTIPLIER); 

	spawnTimerBase = FLOOR(spawnTimerBase, SPAWN_TIMER_BASE_MIN);

	spawnTimerMax = random_s32_in_range_get
	(
		(s32)(spawnTimerBase * SPAWN_TIMER_MIN_MULTIPLIER),
		(s32)(spawnTimerBase * SPAWN_TIMER_MAX_MULTIPLIER)
	);

	return spawnTimerMax;
}

static void
spawn_enemy_set(Enemy* _e, Play* _p)
{
	f64 x;
	f64 y;
	Direction direction;

	direction = (Direction)random_s32_in_range_get(DIRECTION_LEFT, DIRECTION_DOWN);

	x = 0;
	y = 0;

	switch (_e->type)
	{
		case ENEMY_CUPPER:
			switch (direction)
			{
				case DIRECTION_LEFT:
					x = -CUPPER_WIDTH;
					y = random_f64_in_range_get(-CUPPER_HEIGHT, SPAWN_Y_MAX + CUPPER_HEIGHT);
					break;
				case DIRECTION_RIGHT:
					x = SPAWN_X_MAX + CUPPER_WIDTH;
					y = random_f64_in_range_get(-CUPPER_HEIGHT, SPAWN_Y_MAX + CUPPER_HEIGHT);
					break;
				case DIRECTION_UP:
					x = random_f64_in_range_get(-CUPPER_WIDTH, SPAWN_X_MAX + CUPPER_WIDTH);
					y = -CUPPER_HEIGHT;
					break;
				case DIRECTION_DOWN:
					x = random_f64_in_range_get(-CUPPER_WIDTH, SPAWN_X_MAX + CUPPER_WIDTH);
					y = SPAWN_Y_MAX + CUPPER_HEIGHT;
					break;
				default:
					break;
			}
			break;
		case ENEMY_CHIPPER:
			switch (direction)
			{
				case DIRECTION_LEFT:
					x = -CHIPPER_WIDTH;
					y = random_f64_in_range_get(-CHIPPER_HEIGHT, SPAWN_Y_MAX + CHIPPER_HEIGHT);
					break;
				case DIRECTION_RIGHT:
					x = SPAWN_X_MAX + CHIPPER_WIDTH;
					y = random_f64_in_range_get(-CHIPPER_HEIGHT, SPAWN_Y_MAX + CHIPPER_HEIGHT);
					break;
				case DIRECTION_UP:
					x = random_f64_in_range_get(-CHIPPER_WIDTH, SPAWN_X_MAX + CHIPPER_WIDTH);
					y = -CHIPPER_HEIGHT;
					break;
				case DIRECTION_DOWN:
					x = random_f64_in_range_get(-CHIPPER_WIDTH, SPAWN_X_MAX + CHIPPER_WIDTH);
					y = SPAWN_Y_MAX + CHIPPER_HEIGHT;
					break;
				default:
					break;
			}
			break;
		case ENEMY_CHIP:
			switch (direction)
			{
				case DIRECTION_LEFT:
					x = -CHIP_WIDTH;
					y = random_f64_in_range_get(-CHIP_HEIGHT, SPAWN_Y_MAX + CHIP_HEIGHT);
					break;
				case DIRECTION_RIGHT:
					x = SPAWN_X_MAX + CHIP_WIDTH;
					y = random_f64_in_range_get(-CHIP_HEIGHT, SPAWN_Y_MAX + CHIP_HEIGHT);
					break;
				case DIRECTION_UP:
					x = random_f64_in_range_get(-CHIP_WIDTH, SPAWN_X_MAX + CHIP_WIDTH);
					y = -CHIP_HEIGHT;
					break;
				case DIRECTION_DOWN:
					x = random_f64_in_range_get(-CHIP_WIDTH, SPAWN_X_MAX + CHIP_WIDTH);
					y = SPAWN_Y_MAX + CHIP_HEIGHT;
					break;
				default:
					break;
			}
			break;
		case ENEMY_CRUMBLER:
			x = random_f64_in_range_get(CRUMBLER_WIDTH, SPAWN_X_MAX - CRUMBLER_WIDTH);
			y = random_f64_in_range_get(CRUMBLER_HEIGHT, SPAWN_Y_MAX - CRUMBLER_HEIGHT);
			break;
		case ENEMY_NONE:
		default:
			break;
	}

	enemy_set(&_p->enemies, _e, x, y, _p);
}

/* Initializes spawning. */
void
spawn_init(Play* _p)
{
	_p->spawnTimer = SPAWN_TIMER_MAX_DEFAULT;
}

/* Updates spawning. */
void
spawn_update(Play* _p)
{
	_p->spawnTimer--;

	if (!_p->isGracePeriod)
	{
		if (_p->enemies.count == 0)
			_p->spawnTimer = 0;
	}

	if (_p->spawnTimer <= 0)
	{
		Node* node;

		node = list_node_from_index_get(&_p->spawnEnemies, 0);

		if (node)
		{
			Enemy* e;
			EnemyType type;

			type = *(EnemyType*)node->data;

			list_remove(&_p->spawnEnemies, node);

			e = enemy_add(&_p->enemies, type, _p);

			spawn_enemy_set(e, _p);
		}

		_p->spawnTimer = spawn_timer_max_get(_p);
	}
}
