/*
 * DESCRIPTION:
 * Results code.
*/

#include "results.h"

static void results_highest_wave_set(Results* _r);
static void results_cash_held_set(Results* _r);
static void results_cash_collected_set(Results* _r);
static void results_bounties_hunted_set(Results* _r);

/* Highest Wave: X (NG+Y)*/
static void
results_highest_wave_set(Results* _r)
{
	char* s;

	s = malloc(RESULTS_TEXT_MAX);

	if (_r->g->play.game > 1)
	{
		snprintf
		(
			s,
			RESULTS_TEXT_MAX,
			"%s%i(NG+%i)",
			STRING_RESULTS_HIGHEST_WAVE,
			_r->g->play.wave,
			_r->g->play.game
		);

	}
	else
	{
		snprintf
		(
			s,
			RESULTS_TEXT_MAX,
			"%s%i",
			STRING_RESULTS_HIGHEST_WAVE,
			_r->g->play.wave
		);
	}

	text_sprite_init
	(
		&_r->highestWave,
		_r->g->renderer,
		_r->g->fonts[FONT_BIG],
		RESULTS_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	_r->highestWave.x = RESULTS_HIGHEST_WAVE_OFFSET_X;
	_r->highestWave.y = RESULTS_HIGHEST_WAVE_OFFSET_Y;

	free(s);
}

/* Bounties Hunted: X */
static void
results_bounties_hunted_set(Results* _r)
{
	char* s;

	s = malloc(RESULTS_TEXT_MAX);

	snprintf
	(
		s,
		RESULTS_TEXT_MAX,
		"%s%i",
		STRING_RESULTS_BOUNTIES_HUNTED,
		_r->g->play.player.enemiesKilled
	);

	text_sprite_init
	(
		&_r->bountiesHunted,
		_r->g->renderer,
		_r->g->fonts[FONT_BIG],
		RESULTS_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	_r->bountiesHunted.x = RESULTS_BOUNTIES_HUNTED_OFFSET_X;
	_r->bountiesHunted.y = RESULTS_BOUNTIES_HUNTED_OFFSET_Y;

	free(s);
}

/* Cash Collected: $X */
static void
results_cash_collected_set(Results* _r)
{
	char* s;

	s = malloc(RESULTS_TEXT_MAX);

	snprintf
	(
		s,
		RESULTS_TEXT_MAX,
		"%s$%i",
		STRING_RESULTS_CASH_COLLECTED,
		_r->g->play.player.cashCollected
	);

	text_sprite_init
	(
		&_r->cashCollected,
		_r->g->renderer,
		_r->g->fonts[FONT_BIG],
		RESULTS_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	_r->cashCollected.x = RESULTS_CASH_COLLECTED_OFFSET_X;
	_r->cashCollected.y = RESULTS_CASH_COLLECTED_OFFSET_Y;

	free(s);
}

/* Cash Held (On Defeat): $X */
static void
results_cash_held_set(Results* _r)
{
	char* s;

	s = malloc(RESULTS_TEXT_MAX);

	snprintf
	(
		s,
		RESULTS_TEXT_MAX,
		"%s$%i",
		STRING_RESULTS_CASH_HELD,
		_r->g->play.player.cash
	);

	text_sprite_init
	(
		&_r->cashHeld,
		_r->g->renderer,
		_r->g->fonts[FONT_BIG],
		RESULTS_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	_r->cashHeld.x = RESULTS_CASH_HELD_OFFSET_X;
	_r->cashHeld.y = RESULTS_CASH_HELD_OFFSET_Y;

	free(s);
}

/* Initializes results stuff. */
void
results_init(Results* _r, Game* _g)
{
	memset(_r, '\0', sizeof(Results));

	_r->g = _g;

	results_highest_wave_set(_r);
	results_bounties_hunted_set(_r);
	results_cash_collected_set(_r);
	results_cash_held_set(_r);

	text_sprite_init
	(
		&_r->resultsText,
		_r->g->renderer,
		_r->g->fonts[FONT_MEGA],
		RESULTS_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_RESULTS,
		0
	);

	_r->resultsText.x = RESULTS_TEXT_OFFSET_X;
	_r->resultsText.y = RESULTS_TEXT_OFFSET_Y;

	text_sprite_init
	(
		&_r->thanksText,
		_r->g->renderer,
		_r->g->fonts[FONT_MEGA],
		RESULTS_TEXT_COLOR,
		ORIGIN_MIDDLE,
		STRING_RESULTS_THANKS,
		0
	);

	_r->thanksText.x = RESULTS_THANKS_TEXT_OFFSET_X;
	_r->thanksText.y = RESULTS_THANKS_TEXT_OFFSET_Y;

	sprite_init
	(
		&_r->sprite,
		_r->g->textures[TEXTURE_RESULTS],
		ORIGIN_MIDDLE,
		RESULTS_SPRITE_WIDTH,
		RESULTS_SPRITE_HEIGHT
	);

	_r->sprite.x = RESULTS_SPRITE_OFFSET_X;
	_r->sprite.y = RESULTS_SPRITE_OFFSET_Y;

	music_play(_r->g->music[MUSIC_RESULTS], FALSE);
}

/* Frees results stuff. */
void
results_free(Results* _r)
{
	if (_r->highestWave.atlas.texture)
		texture_free(_r->highestWave.atlas.texture);

	if (_r->bountiesHunted.atlas.texture)
		texture_free(_r->bountiesHunted.atlas.texture);

	if (_r->cashHeld.atlas.texture)
		texture_free(_r->cashHeld.atlas.texture);

	if (_r->cashCollected.atlas.texture)
		texture_free(_r->cashCollected.atlas.texture);

	if (_r->thanksText.atlas.texture)
		texture_free(_r->thanksText.atlas.texture);

	if (_r->resultsText.atlas.texture)
		texture_free(_r->thanksText.atlas.texture);
}

/* Updates results stuff. */
void
results_update(Results* _r)
{
	if (mouse_press(&_r->g->mouse, MOUSE_LEFT))
	{
		_r->isEnd = TRUE;

		sound_play(_r->g->sounds[SOUND_MENU], -1);
	}
}

/* Draws results stuff. */
void
results_draw(Results* _r)
{
	SDL_Rect r;

	r.x = 0;
	r.y = 0;
	r.w = RESULTS_RECTANGLE_WIDTH;
	r.h = RESULTS_RECTANGLE_HEIGHT;

        rectangle_draw(&r, _r->g->renderer, RESULTS_BACKGROUND_COLOR);

	sprite_draw(&_r->resultsText, _r->g->renderer);

	sprite_draw(&_r->sprite, _r->g->renderer);

	sprite_draw(&_r->highestWave, _r->g->renderer);
	sprite_draw(&_r->bountiesHunted, _r->g->renderer);
	sprite_draw(&_r->cashCollected, _r->g->renderer);
	sprite_draw(&_r->cashHeld, _r->g->renderer);

	sprite_draw(&_r->thanksText, _r->g->renderer);
}
