/*
 * DESCRIPTION:
 * Handles cutscenes.
*/

#include "scene.h"

static void scene_text_sprite_set(SceneHandler* _h, Game* _g);
static void scene_text_set(SceneHandler* _h, Game* _g);
static void scene_text_end_set(SceneHandler* _h, Game* _g);
static void scene_sprite_set(SceneHandler* _h, Game* _g);

static void
scene_sprite_set(SceneHandler* _h, Game* _g)
{
	sprite_init
	(
		&_h->sprite,
		_g->textures[_h->scenes[_h->sceneIndex].textureType],
		ORIGIN_TOP_LEFT,
		SCENE_SPRITE_WIDTH,
		SCENE_SPRITE_HEIGHT
	);

	_h->sprite.x = SCENE_SPRITE_OFFSET_X;
	_h->sprite.y = SCENE_SPRITE_OFFSET_Y;
}

/* Sets the current text sprite. */
static void
scene_text_sprite_set(SceneHandler* _h, Game* _g)
{
	char* s;

	s = malloc(_h->textIndex + 1);
	memset(s, '\0', _h->textIndex + 1);

	memcpy(s, _h->scenes[_h->sceneIndex].text, _h->textIndex);

	if (_h->text.atlas.texture)
		texture_free(_h->text.atlas.texture);

	text_sprite_init
	(
		&_h->text,
		_g->renderer,
		_g->fonts[FONT_SMALL],
		SCENE_TEXT_COLOR,
		ORIGIN_TOP_LEFT,
		s,
		SCENE_WRAP
	);

	_h->text.x = SCENE_TEXT_OFFSET_X;
	_h->text.y = SCENE_TEXT_OFFSET_Y;

	free(s);
}

/* Sets new text. */
static void
scene_text_set(SceneHandler* _h, Game* _g)
{
	_h->textIndex = 0;
	_h->textLength = strlen(_h->scenes[_h->sceneIndex].text);

	_h->isTextEnd = FALSE;

	scene_text_sprite_set(_h, _g);
}

/* Sets the text of the scene to its last index. */
static void
scene_text_end_set(SceneHandler* _h, Game* _g)
{
	_h->textIndex = _h->textLength;
	_h->isTextEnd = TRUE;
	_h->updateTimer = 0;

	scene_text_sprite_set(_h, _g);
}

/* Initializes the scene. */
void
scene_init(SceneHandler* _h, Game* _g, Scene* _s, s32 _c)
{
	_h->scenes = _s;
	_h->sceneCount = _c;

	_h->sceneIndex = 0;
	_h->isTextEnd = FALSE;
	_h->updateTimer = SCENE_UPDATE_TIMER_MAX;

	scene_sprite_set(_h, _g);
	scene_text_set(_h, _g);
}

/* Frees the scene. */
void
scene_free(SceneHandler* _h, Game* _g)
{
	if (_h->text.atlas.texture)
		texture_free(_h->text.atlas.texture);
}

/* Updates the scene. */
void
scene_update(SceneHandler* _h, Game* _g)
{
	if (!_h->isEnd)
	{
		if (!_h->isTextEnd)
		{
			_h->updateTimer--;

			if (_h->updateTimer <= 0)
			{
				_h->textIndex++;

				scene_text_sprite_set(_h, _g);

				_h->updateTimer = SCENE_UPDATE_TIMER_MAX;

				sound_play(_g->sounds[SOUND_BLIP], 0);
			}
			
			if (mouse_press(&_g->mouse, MOUSE_LEFT))
				scene_text_end_set(_h, _g);

			if (_h->textIndex == _h->textLength)
				_h->isTextEnd = TRUE;
		}
		else
		{
			if (mouse_press(&_g->mouse, MOUSE_LEFT))
			{
				_h->sceneIndex++;
				
				if (_h->sceneIndex >= _h->sceneCount)
					_h->isEnd = TRUE;
				else
				{
					scene_sprite_set(_h, _g);
					scene_text_set(_h, _g);
				}

				sound_play(_g->sounds[SOUND_MENU], 1);
			}
		}

		if (mouse_press(&_g->mouse, MOUSE_RIGHT))
		{
			scene_text_end_set(_h, _g);
			_h->isEnd = TRUE;
		}
	}
}

/* Draws the scene. */
void
scene_draw(SceneHandler* _h, Game* _g)
{
	SDL_Rect r;

	r.x = 0;
	r.y = 0;
	r.w = SCENE_RECTANGLE_WIDTH;
	r.h = SCENE_RECTANGLE_HEIGHT;

	rectangle_draw(&r, _g->renderer, SCENE_BACKGROUND_COLOR);

	sprite_draw(&_h->sprite, _g->renderer);
	
	sprite_draw(&_h->text, _g->renderer);
}


