/*
 * DESCRIPTION:
 * Title code.
*/

#include "title.h"

/* Initializes title. */
void
title_init(Title* _t, Game* _g)
{
	Sprite startButtonText;
	Sprite infoButtonText;
	char* s;

	memset(_t, '\0', sizeof(Title));

	_t->g = _g;

	cursor_init(&_t->cursor, &_t->g->mouse, _t->g->textures[TEXTURE_GUI_CURSOR]);

	sprite_init
	(
		&_t->startBackgroundSprite,
		_t->g->textures[TEXTURE_GUI_SIGN_BIG],
		ORIGIN_MIDDLE,
		TITLE_BUTTON_BACKGROUND_WIDTH,
		TITLE_BUTTON_BACKGROUND_HEIGHT
	);

	sprite_init
	(
		&_t->infoBackgroundSprite,
		_t->g->textures[TEXTURE_GUI_SIGN_BIG],
		ORIGIN_MIDDLE,
		TITLE_BUTTON_BACKGROUND_WIDTH,
		TITLE_BUTTON_BACKGROUND_HEIGHT
	);

	_t->startBackgroundSprite.x = TITLE_START_BUTTON_OFFSET_X;
	_t->startBackgroundSprite.y = TITLE_START_BUTTON_OFFSET_Y;

	_t->infoBackgroundSprite.x = TITLE_INFO_BUTTON_OFFSET_X;
	_t->infoBackgroundSprite.y = TITLE_INFO_BUTTON_OFFSET_Y;

	s = malloc(TITLE_TEXT_MAX);

	snprintf
	(
		s,
		TITLE_TEXT_MAX,
		"%s",
		STRING_TITLE_START
	);

	text_sprite_init
	(
		&startButtonText,
		_t->g->renderer,
		_t->g->fonts[FONT_BIG],
		TITLE_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	snprintf
	(
		s,
		TITLE_TEXT_MAX,
		"%s",
		STRING_TITLE_INFO
	);

	text_sprite_init
	(
		&infoButtonText,
		_t->g->renderer,
		_t->g->fonts[FONT_MEDIUM],
		TITLE_TEXT_COLOR,
		ORIGIN_MIDDLE,
		s,
		0
	);

	free(s);

	startButtonText.rotation = TITLE_TEXT_ROTATION;
	infoButtonText.rotation = TITLE_TEXT_ROTATION;

	button_init(&_t->startButton, &startButtonText);
	button_init(&_t->infoButton, &infoButtonText);

	_t->startButton.sprite.x = _t->startBackgroundSprite.x;
	_t->startButton.sprite.y = _t->startBackgroundSprite.y;

	_t->infoButton.sprite.x = _t->infoBackgroundSprite.x;
	_t->infoButton.sprite.y = _t->infoBackgroundSprite.y;

	sprite_init
	(
		&_t->backgroundSprite,
		_t->g->textures[TEXTURE_TITLE_BACKGROUND],
		ORIGIN_TOP_LEFT,
		TITLE_BACKGROUND_WIDTH,
		TITLE_BACKGROUND_HEIGHT
	);

	_t->backgroundSprite.x = TITLE_BACKGROUND_OFFSET_X;
	_t->backgroundSprite.y = TITLE_BACKGROUND_OFFSET_Y;
	
	sprite_init
	(
		&_t->logoSprite,
		_t->g->textures[TEXTURE_LOGO],
		ORIGIN_MIDDLE,
		TITLE_LOGO_WIDTH,
		TITLE_LOGO_HEIGHT
	);

	_t->logoSprite.x = TITLE_LOGO_OFFSET_X;
	_t->logoSprite.y = TITLE_LOGO_OFFSET_Y;

	music_play(_t->g->music[MUSIC_TITLE], TRUE);
}

/* Frees title. */
void
title_free(Title* _t)
{
	cursor_free(&_t->cursor);
}

/* Updates title. */
void
title_update(Title* _t)
{
	if (_t->isStartPressed)
		_t->isStartPressed = FALSE;

	if (_t->isInfoPressed)
		_t->isInfoPressed = FALSE;

	button_update(&_t->startButton, &_t->g->mouse);
	button_update(&_t->infoButton, &_t->g->mouse);

	if (_t->startButton.state == BUTTON_STATE_PRESS)
	{
		sound_play(_t->g->sounds[SOUND_MENU], -1);
		_t->isStartPressed = TRUE;
	}

	if (_t->infoButton.state == BUTTON_STATE_PRESS)
	{
		sound_play(_t->g->sounds[SOUND_MENU], -1);
		_t->isInfoPressed = TRUE;
	}

	cursor_update(&_t->cursor, &_t->g->mouse);
}

/* Draws title. */
void
title_draw(Title* _t)
{
	sprite_draw(&_t->backgroundSprite, _t->g->renderer);

	sprite_draw(&_t->logoSprite, _t->g->renderer);

	sprite_draw(&_t->startBackgroundSprite, _t->g->renderer);
	sprite_draw(&_t->infoBackgroundSprite, _t->g->renderer);
	
	button_draw(&_t->startButton, _t->g->renderer);
	button_draw(&_t->infoButton, _t->g->renderer);

	cursor_draw(&_t->cursor, _t->g->renderer);
}
