#include "sdl.h"

static void sdl_error(enum SDLErrorType _x);

static void
sdl_error(enum SDLErrorType _x)
{
	switch (_x)
	{
		/* FALLTHROUGH */
		case SDL_ERROR_VIDEO:
		case SDL_ERROR_TIMER:
			printf("%s\n", SDL_GetError());
			break;
		case SDL_ERROR_MIXER:
			printf("%s\n", Mix_GetError());
			break;
		case SDL_ERROR_TTF:
			printf("%s\n", TTF_GetError());
			break;
		case SDL_ERROR_IMAGE:
			printf("%s\n", IMG_GetError());
			break;
		default:
			break;
	}

	sdl_quit();
	exit(1);
}

/* Initializes SDL, SDL_ttf, and SDL_mixer. */
void
sdl_init(void)
{
	if (SDL_INIT_VIDEO < 0) 
		sdl_error(SDL_ERROR_VIDEO);

	if (SDL_INIT_TIMER < 0) 
		sdl_error(SDL_ERROR_TIMER);

	if (TTF_Init() < 0) 
		sdl_error(SDL_ERROR_TTF);

	if (Mix_OpenAudio(0, MIX_DEFAULT_FORMAT, MIX_CHANNELS, 0) < 0) 
		sdl_error(SDL_ERROR_MIXER);
}

/* Quits SDL, SDL_ttf, and SDL_mixer. */
void
sdl_quit(void)
{
	Mix_CloseAudio();
	TTF_Quit();
	SDL_Quit();
}
