#include "list.h"

/* Initialize a list. */
void
list_init(List* _l)
{
	memset(_l, '\0', sizeof(List));

	_l->head = node_get(NULL, 0);
	_l->tail = node_get(NULL, 0);

	_l->head->next = _l->tail;
	_l->head->prev = NULL;

	_l->tail->next = NULL;
	_l->tail->prev = _l->head;

	_l->count = 0;
}

/* Removes a node from the list. */
void
list_remove(List* _l, Node* _n)
{
	Node* next;
	Node* prev;

	next = _n->next;
	prev = _n->prev;

	next->prev = prev;
	prev->next = next;

	node_free(_n);

	_l->count--;
}

/* Removes all the nodes in a given list, but doesn't free the head/tail. */
void
list_clear(List* _l)
{
	Node* node;

	if (_l->count == 0)
		return;

	node = _l->head->next;

	while (node != _l->tail)
	{
		Node* next;

		next = node->next;

		list_remove(_l, node);

		node = next;
	}

	_l->count = 0;
}

/* Frees a given list. */
void
list_free(List* _l)
{
	list_clear(_l);

	node_free(_l->head);
	node_free(_l->tail);

	_l->count = 0;
}

/* Adds a node to the left of another in the list. */
void
list_left_add(List* _l, Node* _a, Node* _b)
{
	_a->next = _b;
        _a->prev = _b->prev;

        _b->prev->next = _a;
        _b->prev = _a;

        _l->count++;
}

/* Adds a node to the right of another in the list. */
void
list_right_add(List* _l, Node* _a, Node* _b)
{
	_a->next = _b->next;
	_a->prev = _b;

	_b->next->prev = _a;
	_b->next = _a;

	_l->count++;
}

/* Appends a new node to the head of a list, and returns its data. */
void*
list_append_new(List* _l, void* _d, size_t _s)
{
	Node* n;

	n = node_get(_d, _s);

	list_right_add(_l, n, _l->head);

	return n->data;
}

/* Gets a malloc'd array of all the data in a list. */
void**
list_data_get(List* _l)
{
	void** data;
	Node* n;
	s32 i;

	if (_l->count == 0)
		return NULL;

	i = 0;
	data = malloc(sizeof(void*) * _l->count);

	n = _l->head->next;

	while (n != _l->tail)
	{
		data[i] = (void*)n->data;
		n = n->next;
		i++;
	}

	return data;
}

/* Returns a node from an index, if it exists. */
Node*
list_node_from_index_get(List* _l, s32 _i)
{
	Node* n;
	s32 i;

	i = 0;
	n = _l->head->next;

	while (n != _l->tail)
	{
		if (i == _i)
			return n;
		n = n->next;

		i++;
	}

	return NULL;
}

/* Gets a node, given the data and if it exists. */
Node*
list_node_from_data_get(List* _l, void* _d, size_t _s)
{
	Node* n;

	n = _l->head->next;

	while (n != _l->tail)
	{
		if (memcmp(n->data, _d, _s) == 0)
			return n;
		n = n->next;
	}

	return NULL;
}

/* Removes a node from the list, given the data. */
void
list_node_from_data_remove(List* _l, void* _d, size_t _s)
{
	Node* n;

	n = list_node_from_data_get(_l, _d, _s);

	if (n)
		list_remove(_l, n);
}
